package cn.jyjz.flowable.config;

import cn.jyjz.flowable.common.SystemConstantsOa;
import cn.jyjz.flowable.domain.vo.FlowModelVO;
import cn.jyjz.flowable.factory.FlowServiceFactory;
import cn.jyjz.flowable.service.IFormCustomService;
import cn.jyjz.xiaoyao.common.base.util.SpringUtils;
import cn.jyjz.xiaoyao.oa.from.service.CandidateuserService;
import org.apache.commons.lang3.StringUtils;
import org.flowable.bpmn.model.BpmnModel;
import org.flowable.bpmn.model.FlowElement;
import org.flowable.bpmn.model.UserTask;
import org.flowable.engine.RepositoryService;
import org.flowable.engine.RuntimeService;
import org.flowable.engine.TaskService;
import org.flowable.engine.delegate.TaskListener;
import org.flowable.engine.runtime.ProcessInstance;
import org.flowable.task.service.delegate.DelegateTask;

import java.util.List;
import java.util.Random;

/**
 * 通过接收页面传来的参数 tostoreid 匹配审批人员
 */
public class TaskStoreCreateListener extends FlowServiceFactory implements TaskListener {
    

    @Override
    public void notify(DelegateTask delegateTask) {
        
        TaskService taskService = SpringUtils.getBean(TaskService.class);
        RepositoryService repositoryService = SpringUtils.getBean(RepositoryService.class);

        CandidateuserService candidateuserService = SpringUtils.getBean(CandidateuserService.class);
        RuntimeService runtimeService = SpringUtils.getBean(RuntimeService.class);

        String processInstanceId = delegateTask.getProcessInstanceId();
        String taskId = delegateTask.getId();
        // 流程实例为空则结束
        if(StringUtils.isBlank(processInstanceId) && StringUtils.isBlank(taskId)){
            return;
        }
        
        //判断节点是否需要跳过，需要判断当前申请人员是否需要过店长
        //获取当前审批节点的usertask
        BpmnModel bpmnModel = repositoryService .getBpmnModel(delegateTask.getProcessDefinitionId());
        //传节点定义key获取当前节点
        FlowElement flowElement = bpmnModel.getFlowElement(delegateTask.getTaskDefinitionKey());
        if(flowElement instanceof UserTask) {
            UserTask userTask = (UserTask) flowElement;
            //如果审批不存在，需要在候选组中查询
            if(StringUtils.isBlank(userTask.getAssignee())){
                //得到候选人
                List<String> candidateUsers = userTask.getCandidateUsers();
                if(null != candidateUsers && !candidateUsers.isEmpty()){
                    Random rand = new Random();
                    taskService.claim(taskId, candidateUsers.get(rand.nextInt(candidateUsers.size())));
                }
                //查询流程实例为空
                ProcessInstance processInstance = runtimeService.createProcessInstanceQuery().processInstanceId(delegateTask.getProcessInstanceId()).singleResult();
                String tostoreid = "";
                //如果流程实例为空，使用表单数据
                if(null == processInstance){
                    tostoreid = delegateTask.getVariables().get("tostoreid").toString();
                }else{
                    //我这里以 流程定义key(oa_category_t定义的关键字，逻辑唯一) + 业务表主键（请假表、调岗表等） + 业务表对应的servicename
                    String businesskey = processInstance.getBusinessKey();
    
                    String[] busskey = businesskey.split(SystemConstantsOa.OA_BUSSINESS_KEY_SPLIT);
                    IFormCustomService iFormCustomService = SpringUtils.getBean(busskey[2]);
                    //设置流程结束状态
                    FlowModelVO flowModelVO = iFormCustomService.getForm(busskey[1]) ;
                    //机构主键
                    tostoreid = flowModelVO.getToStoreid().toString();
                }
                
                //得到候选组
                List<String> candidateGroups = userTask.getCandidateGroups();
                if(null != candidateGroups && !candidateGroups.isEmpty()){
                    String loginuser = null;
                    for(String groupid:candidateGroups){
                        //通过机构主键、候选组，查询相应的人

                        loginuser = candidateuserService.getClaimUserByDeptid(tostoreid,groupid);
                        if(StringUtils.isNotBlank(loginuser)){
                            taskService.claim(taskId, loginuser);
                        }
                    }
                }
            }
        }
    }
}
