package cn.jyjz.flowable.controller;

import cn.jyjz.xiaoyao.common.base.vo.ResultVo;
import cn.jyjz.xiaoyao.common.base.vo.ResultVoUtil;
import cn.jyjz.flowable.service.IFlowDefinitionService;
import cn.jyjz.flowable.utils.PageUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;

/**
 * 流程定义的Controller
 *    流程定义查询
 *    流程部署
 *    删除流程
 *    挂起/激活流程
 *    查看流程图
 */
@Slf4j
@RestController
@RequestMapping("/flow/definition")
public class FlowDefinitionController {

    @Autowired
    private IFlowDefinitionService flowDefinitionService;

    /**
     * 查询流程定义
     * @param pageVo
     * @param model
     * @return
     */
    @GetMapping("/list")
    public ResultVo list(PageUtils pageVo, Model model){
        pageVo = flowDefinitionService.list(pageVo);

        return ResultVoUtil.success(pageVo);
    }

    /**
     * 查询流程定义
     * @param pageVo
     * @param model
     * @return
     */
    @GetMapping("/latestList")
    public ResultVo latestList(PageUtils pageVo, Model model){
        pageVo = flowDefinitionService.latestList(pageVo);

        return ResultVoUtil.success(pageVo);
    }



    /**
     * 部署流程操作
     * @return
     */
    @PostMapping("/deployFlow")
    public ResultVo deployFlow(@RequestParam(value = "name" ,required = false) String name,
                             @RequestParam(value = "categroy",required = false) String categroy,
                             MultipartFile file){
        InputStream inputStream = null;
        try {
            inputStream = file.getInputStream();
            flowDefinitionService.deployFlow(name,categroy,inputStream);
        } catch (IOException e) {
            
            log.error("e={}",e);
            return ResultVoUtil.error();
        }
        return ResultVoUtil.success();
    }

    @PostMapping(value = "/app-repository/deployments",produces = "application/json", consumes = "multipart/form-data")
    @ResponseBody
    public String deployFlowableUIFloe(@RequestParam(value = "deploymentKey") String deploymentKey,
                                       @RequestParam(value = "deploymentName") String deploymentName){
        System.out.println("deploymentKey = " + deploymentKey);
        System.out.println("deploymentName = " + deploymentName);

        return null;
    }


    /**
     * 更新状态
     * @param definitionId
     * @return
     */
    @GetMapping("/updateState")
    public ResultVo updateState(@RequestParam("definitionId") String definitionId){
        flowDefinitionService.updateState(definitionId);
        return ResultVoUtil.success();
    }

    @GetMapping("/deleteFlow/{deployId}")
    public ResultVo deleteFlow(@PathVariable("deployId") String deployId){
        flowDefinitionService.deleteFlow(deployId);
        return ResultVoUtil.success();
    }

    @GetMapping(value = "/flowImg",produces = MediaType.IMAGE_PNG_VALUE)
    @ResponseBody
    public byte[] flowImg(@RequestParam("procDefId") String procDefId){
        byte[] bytes = flowDefinitionService.flowImg(procDefId);
        return bytes;
    }

    @GetMapping(value = "/flowXML",produces = MediaType.TEXT_HTML_VALUE)
    @ResponseBody
    public String flowXML(@RequestParam("procDefId") String procDefId){
        String xml = flowDefinitionService.flowXML(procDefId);

        return xml;
    }

}