package cn.jyjz.flowable.controller;

import cn.jyjz.flowable.common.SystemConstantsOa;
import cn.jyjz.flowable.domain.dto.HistoricTaskInstanceDto;
import cn.jyjz.flowable.domain.vo.FlowTaskVO;
import cn.jyjz.flowable.service.IFlowTaskService;
import cn.jyjz.flowable.utils.PageUtils;
import cn.jyjz.flowable.utils.SysConstant;
import cn.jyjz.xiaoyao.abase.service.SeqManageService;
import cn.jyjz.xiaoyao.admin.dataobject.User;
import cn.jyjz.xiaoyao.admin.service.UserService;
import cn.jyjz.xiaoyao.common.base.config.XiaoyaoConfig;
import cn.jyjz.xiaoyao.common.base.controller.BaseController;
import cn.jyjz.xiaoyao.common.base.param.ParamterPage;
import cn.jyjz.xiaoyao.common.base.service.UserTokenService;
import cn.jyjz.xiaoyao.common.base.util.RequestBaseUtil;
import cn.jyjz.xiaoyao.common.base.util.requestFormat.SearchQuery;
import cn.jyjz.xiaoyao.common.base.vo.ResultVo;
import cn.jyjz.xiaoyao.common.base.vo.ResultVoUtil;
import cn.jyjz.xiaoyao.common.base.vo.UserToken;
import cn.jyjz.xiaoyao.common.mybatisplus.util.SearchQueryFormat;
import cn.jyjz.xiaoyao.oa.from.dataobject.Category;
import cn.jyjz.xiaoyao.oa.from.vo.FlowUnionVo;
import cn.jyjz.xiaoyao.ocr.dataobject.OcrTaskchildPicture;
import cn.jyjz.xiaoyao.ocr.service.OcrTaskchildPictureService;
import com.alanpoi.common.util.StringUtils;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.annotations.*;
import org.flowable.ui.common.model.RemoteUser;
import org.flowable.ui.common.service.idm.RemoteIdmService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Api(tags="ocr_流程接口")
@RestController
@RequestMapping("/flow/task")
public class FlowTaskController extends BaseController {

    @Resource
    IFlowTaskService flowTaskService;
    
    @Resource
    private UserTokenService userTokenService;
    
    @Resource
    private UserService userService;
    @Resource
    private XiaoyaoConfig xiaoyaoConfig;
    @Resource
    private SeqManageService seqManageService;

    @Resource
    private OcrTaskchildPictureService ocrTaskchildPictureService;
//    /**
//     * 查询我的待办任务
//     * @return
//     */
//    @GetMapping("/myToDoTask")
//    @ResponseBody
//    public ResultVo myToDoTask(PageUtils vo, HttpServletRequest request){
//        flowTaskService.myTodoTaskList(vo,request);
//
//        return ResultVoUtil.success(vo);
//    }

    /**
     * 查看过程流程图
     * @param processId
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/flowImg",produces = MediaType.IMAGE_PNG_VALUE)
    @ResponseBody
    public byte[] flowImg(@RequestParam("processId") String processId) throws Exception{
        InputStream diagram = flowTaskService.diagram(processId);
        byte[] bytes = new byte[diagram.available()];
        diagram.read(bytes);
        return bytes;
    }

    /**
     * 查看已办流程图
     * @param processId
     * @return
     * @throws Exception
     */
    @GetMapping(value = "/myFlowImg",produces = MediaType.IMAGE_PNG_VALUE)
    @ResponseBody
    public byte[] myFlowImg(@RequestParam("processId") String processId, HttpServletRequest request) throws Exception{
        InputStream diagram = flowTaskService.myDiagram(processId,request);
        byte[] bytes = new byte[diagram.available()];
        diagram.read(bytes);
        return bytes;
    }
    
    /**
     * 查看表单主键
     * @return
     */
    @PostMapping("/viewFormid")
    @ResponseBody
    public ResultVo viewFormid(@RequestParam("processId") String processId, HttpServletRequest request){
        
        //查询业务主键
        Category category = flowTaskService.viewForm(processId,request);
        if(null != category){
            return ResultVoUtil.success(category);
        }
        return ResultVoUtil.error();
    }

    /**
     * 完成任务
     * @return
     */
    @ApiOperation(value="任务审批", notes="任务审批")
    @PostMapping("/completeFlow")
    @ResponseBody
    public ResultVo complete(FlowTaskVO taskVO,HttpServletRequest request){
        Boolean flag = flowTaskService.complete(taskVO,request);
        return flag?ResultVoUtil.success():ResultVoUtil.error();
    }
    
    /**
     * 批量任务审批
     * @return
     */
    @ApiOperation(value="任务审批", notes="任务审批")
    @PostMapping("/completeBatchFlow")
    @ResponseBody
    public ResultVo completeBatch(HttpServletRequest request){
    
        UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(request));
    
        //任务ID
        String[] taskId = request.getParameterValues("taskId");
        //审批的任务主键
        String[] formid = request.getParameterValues("formid");
    
        String[] taskname = request.getParameterValues("taskname");
        
        //是否同意，true：同意，false：拒绝
        boolean approvd = Boolean.valueOf(request.getParameter("approvd"));
    
        //审批意见")
        String taskComment = request.getParameter("taskComment");
    
        //判断当前任务审批人是否一致
        if(!this.ocrTaskchildPictureService.validateAssignee(formid,userToken)){
            return ResultVoUtil.error("审批人不一致，请到任务审批中处理。");
        }
        
        //TODO  判断当前任务是否审批完毕
        if(!this.ocrTaskchildPictureService.validateApprove(formid)){
            return ResultVoUtil.error("存在已经审批的任务。");
        }
        
        //TODO  判断taskid是否为非法，空置、taskid是否存在
        
        //判断当前任务是否存在历史任务--暂时不做，生成任务时直接保存历史任务的审批工具中

        StringBuffer smg = new StringBuffer();
        for(int i = 0;i < taskId.length;i++){
            String ti = taskId[i];
            String fi = formid[i];
            FlowTaskVO taskVO = new FlowTaskVO();
            taskVO.setTaskId(ti);
            taskVO.setFormid(fi);
            taskVO.setApprovd(approvd);
            taskVO.setTaskComment(taskComment);
    
            Boolean flag = flowTaskService.complete(taskVO,request);
            
            if(!flag){
                if(StringUtils.isNotBlank(smg.toString())){
                    smg.append("，");
                    smg.append(taskname[i]);
                    smg.append("审批失败");
                }else{
                    smg.append(taskname[i]);
                    smg.append("审批失败");
                }

            }
        }
        
        
        
        return (StringUtils.isNotBlank(smg.toString()))?ResultVoUtil.error(smg.toString()):ResultVoUtil.success();
    }
    
    @GetMapping("/listhistore")
    @ResponseBody
    public ResultVo listhistore(PageUtils vo, Model model, HttpServletRequest request){
        String processInstanceId = request.getParameter("processInstanceId");
        List<HistoricTaskInstanceDto> list =  flowTaskService.listHistoriceTask(processInstanceId);
        return ResultVoUtil.success(list);
    }
    /**
     * 我的已办
     *    查询我发起的已经完成的任务
     * @return
     */
    @GetMapping("/finishedList")
    @ResponseBody
    public ResultVo getFinishedList(PageUtils pageUtils, HttpServletRequest request){
        
        SearchQuery searchQuery = this.getParametersStartingWithToSearchJson(request,"search_");
        User user = this.userService.getSessionUser(request);
    
        if(user.isUserIsSys(xiaoyaoConfig.getWebconfig().getSysLoginUser().split(","))){
            flowTaskService.getMyProcessInstances(searchQuery.getQuery(),pageUtils);
        }else{
            searchQuery.addEqual("approveUserid",user.getId().toString());
    
            flowTaskService.getMyProcessInstances(searchQuery.getQuery(),pageUtils);
        }
    
        return ResultVoUtil.success(pageUtils);
    }
    /**
     * 我的发起
     *    查询我发起的已经完成的任务
     * @return
     */
    @ApiOperation(value="待办任务", notes="待办任务")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "header", name = "X-Tenant-Id", value = "租户主键,用户登录时的信息“tenantList”中获得。", dataType = "String", required = true),
    })
    @GetMapping("/listdata")
    @ResponseBody
    public ResultVo<FlowUnionVo> listdata(PageUtils<FlowUnionVo> pageUtils, HttpServletRequest request){
        SearchQuery searchQuery = this.getParametersStartingWithToSearchJson(request,"search_");
    
        //租户主键，由前端页面传送
        String tenantId = request.getHeader("X-Tenant-Id");
    
        if(org.apache.commons.lang3.StringUtils.isBlank(tenantId)){
            return ResultVoUtil.error("租户主键不能为空");
        }

        User user = this.userService.getSessionUser(request);
        if(user.isUserIsSys(xiaoyaoConfig.getWebconfig().getSysLoginUser().split(","))){
            
            flowTaskService.getMyProcessInstances(searchQuery.getQuery(),pageUtils);
        }else{
            String assigneeId = request.getParameter("assigneeId");
            //如果assigneeId 为空查询发起流程，否则查询待办流程
            if(StringUtils.isNotBlank(assigneeId) && assigneeId.equals(user.getId().toString())){
                searchQuery.addEqual("assignee",user.getLoginname());
                searchQuery.addEqual("states",SystemConstantsOa.OA_STATUS_TYPE_ING.toString());
            }else{
                searchQuery.addEqual("createuser",user.getLoginname());
            }
            searchQuery.addEqual("tenantid",tenantId);
            flowTaskService.getMyProcessInstances(searchQuery.getQuery(),pageUtils);
            
            //待办信息，增加是否查看标记
            if(StringUtils.isNotBlank(assigneeId) && assigneeId.equals(user.getId().toString())){
    
                List list = pageUtils.getList();
                if(null != list && !list.isEmpty()){
                    int count = list.size();
                    for(int i = 0;i < count;i++){
                        FlowUnionVo flowUnionVo = (FlowUnionVo) list.get(i);
                        //flowUnionVo.setView(userViewApproeService.haveByFromid(flowUnionVo.getFormid().toString()));
                    }
                }

            }
            
        }
        return ResultVoUtil.success(pageUtils);
    }
    
    /**
     * 终审列表
     * @return
     */
    @ApiOperation(value="我的终审列表", notes="我的终审列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "header", name = "X-Tenant-Id", value = "租户主键,用户登录时的信息“tenantList”中获得。", dataType = "String", required = true),
    })
    @GetMapping("/listfinal")
    @ResponseBody
    public ResultVo<OcrTaskchildPicture> listFinalData(PageUtils<OcrTaskchildPicture> pageUtils, HttpServletRequest request){
        SearchQuery searchQuery = this.getParametersStartingWithToSearchJson(request,"search_");
        
        //租户主键，由前端页面传送
        String tenantId = request.getHeader("X-Tenant-Id");
        
        if(org.apache.commons.lang3.StringUtils.isBlank(tenantId)){
            return ResultVoUtil.error("租户主键不能为空");
        }
        
        User user = this.userService.getSessionUser(request);
        if(user.isUserIsSys(xiaoyaoConfig.getWebconfig().getSysLoginUser().split(","))){
            
            flowTaskService.getFinalProcessInstances(searchQuery.getQuery(),pageUtils);
        }else{
    
            searchQuery.addEqual("thisLoginUserid",user.getId().toString());
            searchQuery.addEqual("tenantid",tenantId);
            //TODO 增加终审关联表查询 ，增加排序字段传输
            flowTaskService.getFinalProcessInstances(searchQuery.getQuery(),pageUtils);
            
        }
        return ResultVoUtil.success(pageUtils);
    }
    
    /**
     * 我的发起
     *    查询我发起的已经完成的任务
     *    （CATEGORYID）流程类型为必填项
     * @return
     */
    @GetMapping("/listdataall")
    @ResponseBody
    public ResultVo listdataAll(HttpServletRequest request) {
        SearchQuery searchQuery = this.getParametersStartingWithToSearchJson(request, "search_");
        List<FlowUnionVo> flowUnionVoList = null;
        String categoryid = request.getParameter("categoryid");
        String buessinessno = request.getParameter("buessinessno");
        
        //业务表id
        String formid = request.getParameter("viewformid");
        if (StringUtils.isNotBlank(categoryid)) {
            
            User user = this.userService.getSessionUser(request);
            if (user.isUserIsSys(xiaoyaoConfig.getWebconfig().getSysLoginUser().split(","))) {
                flowUnionVoList = flowTaskService.getMyProcessInstances(searchQuery.getQuery());
            } else {
                searchQuery.addEqual("createuser", user.getLoginname());
                //流程类型，对应流程类型表（OA_CATEGORY_T）的id
                searchQuery.addEqual("categoryid", categoryid);
                if(StringUtils.isNotBlank(buessinessno)){
                    searchQuery.addEqual("buessinessno", buessinessno);
                }
                if(StringUtils.isNotBlank(formid)){
                    searchQuery.addEqual("id", formid);
                }
                
                
                searchQuery.addEqual("states", SystemConstantsOa.OA_STATUS_TYPE_END.toString());
                flowUnionVoList = flowTaskService.getMyProcessInstances(searchQuery.getQuery());
            }
        }
        return ResultVoUtil.success(flowUnionVoList);
    }
    /**
     * 抄送给我的
     * @param pageUtils
     * @param request
     * @return
     */
    @GetMapping("/listdatacc")
    @ResponseBody
    public ResultVo listdatacc(ParamterPage paramterPage,PageUtils pageUtils, HttpServletRequest request){
        SearchQuery searchQuery = this.getParametersStartingWithToSearchJson(request, "search_");
        Map<String, Object> query = searchQuery.getQuery();
        
        UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(request));
        
        flowTaskService.getMyListCc(query,userToken,paramterPage);
        
        return ResultVoUtil.success(pageUtils);
    }
    /**
     * 抄送我的
     *    查询我发起的已经完成的任务
     * @return
     */
    @GetMapping("/mylistcc")
    @ResponseBody
    public ResultVo myListCc(ParamterPage paramterPage, HttpServletRequest request){
        UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(request));
        SearchQuery searchQuery = this.getParametersStartingWithToSearchJson(request, "search_");
        Map<String, Object> query = searchQuery.getQuery();
        IPage<FlowUnionVo> flowableccIPage = flowTaskService.getMyListCc(query,userToken,paramterPage);
        
        return ResultVoUtil.success(flowableccIPage);
    }
    /**
     * 撤销流程
     * @return
     */
    @GetMapping("/cancelTask")
    @ResponseBody
    public ResultVo cancelTask(HttpServletRequest request){
        String taskId = request.getParameter("taskId");
        String reason = request.getParameter("reason");
        flowTaskService.cancelTask(taskId,reason);

        return ResultVoUtil.success();
    }
    
    /**
     * 重置流程流程
     * @return
     */
    @ApiOperation(value="重置流程流程", notes="重置流程流程")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "header", name = "X-Tenant-Id", value = "租户主键,用户登录时的信息“tenantList”中获得。", dataType = "String", required = true),
            @ApiImplicitParam(paramType = "path",name = "fromid", value = "任务表主键", required = true)
    })
    @GetMapping("/removeTask")
    @ResponseBody
    public ResultVo removeTask(HttpServletRequest request){
        String fromid = request.getParameter("fromid");
        String taskid = request.getParameter("taskid");
        return flowTaskService.removeTask(fromid,taskid);
        
        //return ResultVoUtil.success();
    }
}
