package cn.jyjz.flowable.modeler.services;

import cn.jyjz.xiaoyao.admin.dataobject.Roles;
import cn.jyjz.xiaoyao.admin.dataobject.User;
import cn.jyjz.xiaoyao.admin.dataobject.Userroles;
import cn.jyjz.xiaoyao.admin.service.RolesService;
import cn.jyjz.xiaoyao.admin.service.UserService;
import cn.jyjz.xiaoyao.admin.service.UserrolesService;
import cn.jyjz.xiaoyao.oa.from.dataobject.Candidate;
import cn.jyjz.xiaoyao.oa.from.service.CandidateService;
import org.flowable.ui.common.model.RemoteGroup;
import org.flowable.ui.common.model.RemoteToken;
import org.flowable.ui.common.model.RemoteUser;
import org.flowable.ui.common.service.idm.RemoteIdmService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service
public class MyRemoteServiceImpl implements RemoteIdmService {

    private Logger LOGGER = LoggerFactory.getLogger(MyRemoteServiceImpl.class);

    @Autowired
    UserService userService;
    
    @Autowired
    private CandidateService candidateService;
    @Override
    public RemoteUser authenticateUser(String username, String password) {
        LOGGER.warn("MyRemoteServiceImpl:authenticateUser");
        System.out.println("username = " + username);
        return null;
    }

    @Override
    public RemoteToken getToken(String tokenValue) {
        LOGGER.warn("MyRemoteServiceImpl:getToken");
        System.out.println("tokenValue = " + tokenValue);
        return null;
    }



    @Override
    public RemoteUser getUser(String userId) {
        User user = userService.findByLoginname(userId);
        RemoteUser remoteUser = new RemoteUser();
        remoteUser.setId(user.getLoginname()+"");
        remoteUser.setFirstName(user.getUsername());
        remoteUser.setFullName(user.getUsername());
        // 根据当前的用户查询到对应的角色
        Userroles userRole = new Userroles();
        userRole.setUserid(user.getId());
        // 用户和角色的关系
        List<Candidate> userCandidateList = candidateService.listCandiateByUserid(user.getId());
        List<RemoteGroup> remoteGroups = userCandidateList.stream().map(item -> {
            RemoteGroup group = new RemoteGroup();
            
            group.setId(item.getId() + "");
            group.setName(item.getCandidatename());
            
            return group;
        }).collect(Collectors.toList());
        remoteUser.setGroups(remoteGroups);
        return remoteUser;
    }


    /**
     * 分配用户或者候选人
     * @param filter
     * @return
     */
    @Override
    public List<RemoteUser> findUsersByNameFilter(String filter) {
        LOGGER.warn("MyRemoteServiceImpl:findUsersByNameFilter");
        List<User> list = userService.selectUserLikeName(filter);
        return list.stream().map(item->{
            RemoteUser remoteUser = new RemoteUser();
            remoteUser.setId(item.getLoginname());
            remoteUser.setFirstName(item.getUsername());
            remoteUser.setFullName(item.getUsername());
            remoteUser.setTenantId(item.getId().toString());
            return remoteUser;
        }).collect(Collectors.toList());
    }


    /**
     * 根据角色编号查询到所有的用户信息
     * @param groupId
     * @return
     */
    @Override
    public List<RemoteUser> findUsersByGroup(String groupId) {
        List<User> list = userService.selectRolesUser(Long.parseLong(groupId));
        return list.stream().map(user->{
            RemoteUser remoteUser = new RemoteUser();
            remoteUser.setId(user.getLoginname());
            remoteUser.setFirstName(user.getUsername());
            remoteUser.setFullName(user.getUsername());
            return remoteUser;
        }).collect(Collectors.toList());
    }

    /**
     *  org.flowable.ui.modeler.rest.app.EditorGroupsResource#getGroups(java.lang.String)
     *  url: http://localhost:8081/flowable-modeler-demo/app/rest/editor-groups

     * @param groupId
     * @return
     */
    @Override
    public RemoteGroup getGroup(String groupId) {
        LOGGER.warn("MyRemoteServiceImpl:getGroup");
        Candidate candidate = candidateService.selectDtoById(Long.parseLong(groupId));
        RemoteGroup group = new RemoteGroup();
        group.setId(candidate.getId().toString());
        group.setName(candidate.getCandidatename());
        return group;
    }


    /**
     *  分配用户功能
     *   组对应于 角色
     *  http://localhost:8081/flowable-modeler-demo/app/rest/editor-groups
     *  org.flowable.ui.modeler.rest.app.EditorGroupsResource#getGroups(java.lang.String)
     * @param filter
     * @return
     */
    @Override
    public List<RemoteGroup> findGroupsByNameFilter(String filter) {
        LOGGER.warn("MyRemoteServiceImpl:findGroupsByNameFilter");
        // 根据查询的条件查询出相关的角色信息
        List<Candidate> list = candidateService.listCandiateByKey(filter);
        return list.stream().map(item ->{
            RemoteGroup group = new RemoteGroup();
            group.setId(item.getId().toString());
            group.setName(item.getCandidatename());
            return group;
        }).collect(Collectors.toList());
    }
}
