package cn.jyjz.flowable.service.impl;


//import cn.jyjz.flowable.convert.JSONConvertHtml;
//import cn.jyjz.flowable.convert.flowableui.FlowableUIJSONConvertHtml;
import cn.jyjz.flowable.domain.dto.FlowDeployProcDto;
import cn.jyjz.flowable.common.ProcessConstant;
import cn.jyjz.flowable.entity.DeModel;
import cn.jyjz.flowable.entity.FormDefinition;
import cn.jyjz.flowable.factory.FlowServiceFactory;
import cn.jyjz.flowable.service.IDeModelService;
import cn.jyjz.flowable.service.IFlowDefinitionService;
import cn.jyjz.flowable.service.IFormDefinitionService;
import cn.jyjz.flowable.utils.PageUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;

import org.apache.commons.io.IOUtils;

import org.flowable.bpmn.model.FormProperty;
import org.flowable.bpmn.model.UserTask;

import org.flowable.engine.repository.Deployment;
import org.flowable.engine.repository.ProcessDefinition;
import org.flowable.engine.repository.ProcessDefinitionQuery;
import org.flowable.engine.runtime.ProcessInstance;

import org.flowable.task.api.Task;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

/**
 * 流程定义相关的Service
 */
@Service
public class FlowDefinitionServiceImpl extends FlowServiceFactory implements IFlowDefinitionService {
    
    @Autowired
    IFormDefinitionService formDefinitionService;
    @Autowired
    IDeModelService deModelService;
    @Override
    public PageUtils list(PageUtils pageVo) {
        List<FlowDeployProcDto> list = new ArrayList<>();
        // 1.查询所有流程定义
        ProcessDefinitionQuery processDefinitionQuery = repositoryService.createProcessDefinitionQuery();
        if(StringUtils.isNotBlank(pageVo.getKey())){
            processDefinitionQuery.processDefinitionNameLike("%"+pageVo.getKey()+"%");
        }

        //查询到的分页的数据
        List<ProcessDefinition> processDefinitions = processDefinitionQuery.
                listPage(pageVo.getPageSize() * (pageVo.getCurrPage() - 1), pageVo.getPageSize());

        if(processDefinitions!=null && processDefinitions.size() > 0){
            // 说明有部署的流程
            for (ProcessDefinition processDefinition : processDefinitions) {
                FlowDeployProcDto dto = new FlowDeployProcDto();
                // 根据部署的流程ID查找对应的 流程定义信息
                Deployment deployment = repositoryService
                        .createDeploymentQuery().deploymentId(processDefinition.getDeploymentId()).singleResult();

                dto.setId(processDefinition.getId());
                dto.setName(processDefinition.getName());
                dto.setCategory(processDefinition.getCategory());
                dto.setTenantId(processDefinition.getTenantId());
                dto.setDeployId(processDefinition.getDeploymentId());
                dto.setFlowKey(processDefinition.getKey());
                dto.setDeployDate(deployment.getDeploymentTime());
                dto.setDescription(processDefinition.getDescription());
                dto.setVersion(processDefinition.getVersion());
                dto.setSuspensionState(processDefinition.isSuspended()?2:1);
                list.add(dto);
            }
        }

        pageVo.setList(list);
        pageVo.setTotalCount((int)processDefinitionQuery.count());
        // 4 / 5 1  5/5 2  6/5 2
        pageVo.setTotalPage((int)((processDefinitionQuery.count()+1) / pageVo.getPageSize() + 1));
        return pageVo;
    }

    /**
     * 更新流程的状态
     * @param definitionId
     */
    @Override
    public void updateState(String definitionId) {
        ProcessDefinition processDefinition = repositoryService.createProcessDefinitionQuery().processDefinitionId(definitionId).singleResult();
        if(processDefinition.isSuspended()){
            // 表示是挂起状态 --> 更新为激活状态
            repositoryService.activateProcessDefinitionById(definitionId,true,null);
        }else{
            // 表示是激活状态 --> 挂起状态
            repositoryService.suspendProcessDefinitionById(definitionId,true,null);
        }
    }

    /**
     * 根据流程定义编号来删除对应流程
     * @param deployId
     */
    @Override
    public void deleteFlow(String deployId) {
        // 第二个参数 true表示级联删除 这个流程 对应的 实例  任务 历史数据 都会被删除掉
        repositoryService.deleteDeployment(deployId,true);
    }

    /**
     * 部署流程
     * @param name
     * @param categroy
     * @param inputStream
     */
    @Override
    public void deployFlow(String name, String categroy, InputStream inputStream) {
        repositoryService.createDeployment()
                .addInputStream(name+ ProcessConstant.BPMN_FILE_SUFFIX,inputStream)
                .name(name)
                .category(categroy)
                .deploy();

    }

    @Override
    public byte[] flowImg(String procDefId) {
    
        //获取流程图
//        BpmnModel bpmnModel = repositoryService.getBpmnModel(procDefId);
//        ProcessEngineConfiguration engconf = processEngine.getProcessEngineConfiguration();
//        ProcessDiagramGenerator diagramGenerator = engconf.getProcessDiagramGenerator();
//
//
//        diagramGenerator.
        ProcessDefinition processDefinition = repositoryService.createProcessDefinitionQuery().processDefinitionId(procDefId).singleResult();
        // 读取到的流程图 字节数据源
        try {
            
            String resourceName = processDefinition.getDiagramResourceName();
            if(StringUtils.isBlank(resourceName)){
                resourceName = processDefinition.getResourceName();
            }
            
            InputStream in = repositoryService.getResourceAsStream(processDefinition.getDeploymentId(),resourceName);
            byte[] b = new byte[in.available()];
            in.read(b);
            return b;
        }catch (IOException e){

        }
        return null;
    }

    @Override
    public String flowXML(String procDefId) {
        ProcessDefinition processDefinition = repositoryService.createProcessDefinitionQuery().processDefinitionId(procDefId).singleResult();
        // 读取到的流程图 字节数据源
        try {
            InputStream in = repositoryService.getResourceAsStream(processDefinition.getDeploymentId(),processDefinition.getResourceName());
            return IOUtils.toString(in, StandardCharsets.UTF_8);
        }catch (IOException e){

        }
        return null;
    }

    @Override
    public PageUtils latestList(PageUtils pageVo) {
        List<FlowDeployProcDto> list = new ArrayList<>();
        // 1.查询所有部署的流程
        ProcessDefinitionQuery processDefinitionQuery = repositoryService.createProcessDefinitionQuery();

        if(StringUtils.isNotBlank(pageVo.getKey())){
            // 说明需要根据条件查询
            processDefinitionQuery.processDefinitionNameLike("%"+pageVo.getKey()+"%");
        }
        //查询到的分页的数据
        List<ProcessDefinition> processDefinitions = processDefinitionQuery.
                latestVersion().// 找到最新的流程定义信息
                        listPage(pageVo.getPageSize() * (pageVo.getCurrPage() - 1), pageVo.getPageSize());

        if(processDefinitions!=null && processDefinitions.size() > 0){
            // 说明有部署的流程
            for (ProcessDefinition processDefinition : processDefinitions) {
                FlowDeployProcDto dto = new FlowDeployProcDto();
                // 根据部署的流程ID查找对应的 流程定义信息
                Deployment deployment = repositoryService
                        .createDeploymentQuery()
                        .deploymentId(processDefinition.getDeploymentId())
                        .singleResult();
    
                ProcessInstance processInstance = runtimeService.startProcessInstanceById(processDefinition.getId());
                Task task = taskService.createTaskQuery().processInstanceId(processInstance.getProcessInstanceId()).singleResult();
    
                
                //TODO 如果是其他表单，另外使用
//                if(null != task && StringUtils.isNotBlank(task.getFormKey())){
//
//                    List<DeModel> modelList = deModelService.list(task.getFormKey(), 2);
//
//                    if(null != modelList){
//                        DeModel item = modelList.get(0);
//                        String modelEditorJson = item.getModelEditorJson();
//                        JSONConvertHtml convertHtml = new FlowableUIJSONConvertHtml();
//                        String html = convertHtml.convertHtml(modelEditorJson);
//                        item.setHtml(html);
//                    }
//
//
//                }

                
                dto.setId(processDefinition.getId());
                // deployment.getName()  流程部署的名称
                // processDefinition.getName() 流程定义的名称
                dto.setName(processDefinition.getName());
                dto.setCategory(processDefinition.getCategory());
                dto.setTenantId(processDefinition.getTenantId());
                dto.setDeployId(processDefinition.getDeploymentId());
                dto.setFlowKey(processDefinition.getKey());
                dto.setDeployDate(deployment.getDeploymentTime());
                dto.setDescription(processDefinition.getDescription());
                dto.setVersion(processDefinition.getVersion());
                dto.setSuspensionState(processDefinition.isSuspended()?2:1);
                list.add(dto);
            }
        }

        pageVo.setList(list);
        pageVo.setTotalCount((int)processDefinitionQuery.count());
        // 4 / 5 1  5/5 2  6/5 2
        pageVo.setTotalPage((int)((processDefinitionQuery.count()+1) / pageVo.getPageSize() + 1));
        return pageVo;
    }

}