package cn.jyjz.flowable.service.impl;

import cn.jyjz.flowable.common.SystemConstantsOa;
import cn.jyjz.flowable.domain.vo.FlowModelVO;
import cn.jyjz.flowable.domain.vo.FlowTaskCompleteVO;
import cn.jyjz.flowable.factory.FlowServiceFactory;
import cn.jyjz.flowable.service.IFlowInstanceService;
import cn.jyjz.flowable.service.IFlowTaskService;
import cn.jyjz.flowable.service.IFormCustomService;
import cn.jyjz.xiaoyao.common.base.config.XiaoyaoConfig;
import cn.jyjz.xiaoyao.common.base.exception.ServiceException;
import cn.jyjz.xiaoyao.common.base.language.SystemMessage;
import cn.jyjz.xiaoyao.common.base.service.UserTokenService;
import cn.jyjz.xiaoyao.common.base.util.RequestBaseUtil;
import cn.jyjz.xiaoyao.common.base.util.SpringUtils;
import cn.jyjz.xiaoyao.common.base.util.StringUtils;
import cn.jyjz.xiaoyao.common.base.vo.UserToken;
import cn.jyjz.xiaoyao.framework.security.model.AdminUserModel;
import cn.jyjz.xiaoyao.framework.security.service.FlowableUserLoginService;
import cn.jyjz.xiaoyao.oa.from.service.UserViewapproeService;
import lombok.extern.slf4j.Slf4j;
import org.flowable.bpmn.model.*;
import org.flowable.engine.RepositoryService;
import org.flowable.engine.impl.bpmn.behavior.ParallelMultiInstanceBehavior;
import org.flowable.engine.repository.ProcessDefinition;
import org.flowable.engine.runtime.ProcessInstance;
import org.flowable.task.api.Task;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class FlowInstanceServiceImpl extends FlowServiceFactory implements IFlowInstanceService {
    @Resource
    private XiaoyaoConfig xiaoyaoConfig;
    @Resource
    private UserTokenService userTokenService;
    @Resource
    protected RepositoryService repositoryService;
    @Resource
    private IFlowTaskService flowTaskService;
    @Resource
    private UserViewapproeService userViewApproeService;
    @Override
    public boolean startProcessInstanceById(String procDefId, Map<String, Object> variables, HttpServletRequest request,String formid,String servicename) {
        
        IFormCustomService iFormCustomService = null;
        if(StringUtils.isNotBlank(servicename)){
            iFormCustomService = SpringUtils.getBean(servicename);
        }
        
        if(StringUtils.isBlank(formid) ){
            //表单主键
            formid = iFormCustomService.saveForm(request);
        }
        
        UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(request));

        
        String userName = userToken.getLoginname();
        // 绑定当前认证的账号  我们的认证系统如果使用的是 SpringSecurity的话，默认就会设置当前登录的账号
        //identityService.setAuthenticatedUserId(userName);
        if(null == variables){
            variables = new HashMap<>();
        }
        variables.put("loginUser",userName);
        //根据流程定义信息
        ProcessDefinition processDefinition = repositoryService.createProcessDefinitionQuery().processDefinitionId(procDefId).singleResult();
        
        
        //业务key 用于待办已办 扩展字段 展示使用
        //我这里以 流程定义key(oa_category_t定义的关键字，逻辑唯一) + 业务表主键（请假表、调岗表等） + 业务表对应的servicename
        String businessKey= userToken.getLoginname() + SystemConstantsOa.OA_BUSSINESS_KEY_SPLIT+ formid + SystemConstantsOa.OA_BUSSINESS_KEY_SPLIT+ servicename;
        
        ProcessInstance processInstance = null;
        
        //启动了一个流程实例
        try {
            
            variables.put("businessKey",businessKey);
            processInstance = runtimeService.startProcessInstanceById(procDefId,businessKey,variables) ;
            
            if(null != iFormCustomService){
                FlowTaskCompleteVO flowTaskCompleteVO = flowTaskService.updateNowTaskAssignee(processInstance.getProcessInstanceId(),processInstance.getProcessDefinitionId());
                
                if(null == flowTaskCompleteVO){
                    throw new ServiceException("没有审批人，请联系管理员，流程定义：" + businessKey);
                }
                //保存instanceId到数据库，需要根据流程分类进行，判断更新
                boolean isupdate = iFormCustomService.updateForm(formid,processDefinition.getDeploymentId(),processDefinition.getId(),flowTaskCompleteVO.getAssignee(),flowTaskCompleteVO.getTaskId(),processInstance.getProcessInstanceId());
                if(isupdate){
                    FlowModelVO flowModelVO = iFormCustomService.getForm(formid);
                    //增加未查看记录
                    userViewApproeService.saveUserViewApproe(flowTaskCompleteVO.getAssignee(),formid,flowModelVO.getTenantid().toString());
                }
                
            }
            
            log.info("流程启动成功:procDefId:{}",procDefId);
            return true;
        }catch (Exception e){
            log.error("流程启动失败:procDefId:{}",procDefId);
            log.error("流程启动失败:e:{}",e);
            
            //删除流程
            if(null != processInstance){
                runtimeService.deleteProcessInstance(processInstance.getProcessInstanceId(), SystemMessage.SYS_OA_START_ERROR);
                historyService.deleteHistoricProcessInstance(processInstance.getProcessInstanceId());
            }
            if(StringUtils.isNotBlank(servicename) && StringUtils.isNotBlank(formid)){
                iFormCustomService.delForm(formid);
            }
        }
        return false;
    }
    
    @Override
    public boolean startProcessInstanceById(String procDefId, String userName,String createuser, String formid,String servicename) {
        
        IFormCustomService iFormCustomService = null;
        if(StringUtils.isNotBlank(servicename)){
            iFormCustomService = SpringUtils.getBean(servicename);
        }
        
        if(StringUtils.isBlank(formid) ){
            //表单主键
            return false;
        }

        Map<String, Object> variables = new HashMap<>();
        variables.put("loginUser",userName);
        //根据流程定义信息
        ProcessDefinition processDefinition = repositoryService.createProcessDefinitionQuery().processDefinitionId(procDefId).singleResult();
        
        
        //业务key 用于待办已办 扩展字段 展示使用
        //我这里以 流程定义key(oa_category_t定义的关键字，逻辑唯一) + 业务表主键（请假表、调岗表等） + 业务表对应的servicename
        String businessKey= createuser + SystemConstantsOa.OA_BUSSINESS_KEY_SPLIT+ formid + SystemConstantsOa.OA_BUSSINESS_KEY_SPLIT+ servicename;
        
        ProcessInstance processInstance = null;
        
        //启动了一个流程实例
        try {
            
            variables.put("businessKey",businessKey);
            processInstance = runtimeService.startProcessInstanceById(procDefId,businessKey,variables) ;
            
            if(null != iFormCustomService){
                FlowTaskCompleteVO flowTaskCompleteVO = flowTaskService.updateNowTaskAssignee(processInstance.getProcessInstanceId(),processInstance.getProcessDefinitionId());
                
                if(null == flowTaskCompleteVO){
                    throw new ServiceException("没有审批人，请联系管理员，流程定义：" + businessKey);
                }
                //保存instanceId到数据库，需要根据流程分类进行，判断更新
                boolean isupdate = iFormCustomService.updateForm(formid,processDefinition.getDeploymentId(),processDefinition.getId(),flowTaskCompleteVO.getAssignee(),flowTaskCompleteVO.getTaskId(),processInstance.getProcessInstanceId());
                if(isupdate){
                    FlowModelVO flowModelVO = iFormCustomService.getForm(formid);
                    //增加未查看记录
                    userViewApproeService.saveUserViewApproe(flowTaskCompleteVO.getAssignee(),formid,flowModelVO.getTenantid().toString());
                }
                
            }
            
            log.info("流程启动成功:procDefId:{}",procDefId);
            return true;
        }catch (Exception e){
            log.error("流程启动失败:procDefId:{}",procDefId);
            log.error("流程启动失败:e:{}",e);
            
            //删除流程
            if(null != processInstance){
                runtimeService.deleteProcessInstance(processInstance.getProcessInstanceId(), SystemMessage.SYS_OA_START_ERROR);
                historyService.deleteHistoricProcessInstance(processInstance.getProcessInstanceId());
            }
            if(StringUtils.isNotBlank(servicename) && StringUtils.isNotBlank(formid)){
                iFormCustomService.delForm(formid);
            }
        }
        return false;
    }
}
