package cn.jyjz.xiaoyao.oa.from.controller;

import java.io.IOException;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.jyjz.xiaoyao.admin.service.TenantService;
import cn.jyjz.xiaoyao.common.base.util.RequestBaseUtil;
import cn.jyjz.xiaoyao.common.base.vo.UserToken;
import cn.jyjz.xiaoyao.common.mybatisplus.util.SearchQueryFormat;
import cn.jyjz.xiaoyao.oa.from.dataobject.Category;
import cn.jyjz.xiaoyao.oa.from.service.CategoryService;
import com.alibaba.fastjson.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;

import cn.jyjz.xiaoyao.admin.service.UserService;
import cn.jyjz.xiaoyao.common.base.service.UserTokenService;
import cn.jyjz.xiaoyao.common.base.controller.BaseController;
import cn.jyjz.xiaoyao.common.base.param.ParamterPage;
import cn.jyjz.xiaoyao.common.base.util.requestFormat.SearchQuery;
import cn.jyjz.xiaoyao.common.base.vo.ResultVo;
import cn.jyjz.xiaoyao.common.base.vo.ResultVoUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;

@RestController
@RequestMapping("/backstage/oa/category")
public class CategoryController extends BaseController{

	private Logger log = LoggerFactory.getLogger(this.getClass());

	@Resource
	private CategoryService categoryService;
	@Resource
	private TenantService tenantService;
	@Resource
	private UserTokenService userTokenService;
	@Resource
	private UserService userService;
	/**
	*方法描述：返回json字符串，接受参数，dto名称以及分页信息
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2023-04-19 14:41:09
	**/
	@GetMapping("/list")
	public ResultVo<IPage<Category>> listCategory(ParamterPage paramterPage, HttpServletRequest request, HttpServletResponse response) throws IOException{
		SearchQuery searchQuery = this.getParametersStartingWithToSearchJson(request, "search_");
		Category category = new Category();
		QueryWrapper<Category> queryWrapper = SearchQueryFormat.queryStringFormat(searchQuery,category);
		IPage<Category> page = this.categoryService.selectSearchListPage(paramterPage,queryWrapper);
		for(Category category1:page.getRecords()){
			category1.setTenant(tenantService.selectDtoById(category1.getTenantid()));
		}
		return ResultVoUtil.success(page);
	}

	/**
	*方法描述：保存添加数据
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2023-04-19 14:41:09
	**/
	@PostMapping("/save")
	public ResultVo saveCategory(HttpServletResponse response,HttpServletRequest request,Category tab) throws IOException{
		UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(request));
		
		//查询流程定义id不为空的数据
		Category category = this.categoryService.getCategoryNotNull();
		tab.setProcessdefinitionid(category.getProcessdefinitionid());
		
		tab.setCreateuser(userToken.getLoginname());
		tab.setCreatedate(System.currentTimeMillis());
		this.categoryService.save(tab);
		return ResultVoUtil.success();
	}

	/**
	*方法描述：根据页面form传来的对象修改数据
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2023-04-19 14:41:09
	**/
	@PostMapping("/edit")
	public ResultVo editCategory(HttpServletResponse response,HttpServletRequest request,Category tab) throws IOException{
		UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(request));
		
		//查询流程定义id不为空的数据
		Category category = this.categoryService.getCategoryNotNull();
		tab.setProcessdefinitionid(category.getProcessdefinitionid());
		
		tab.setUpdateuser(userToken.getLoginname());
		tab.setUpdatetime(System.currentTimeMillis());
		this.categoryService.updateById(tab);
		return ResultVoUtil.success();
	}
	
	/**
	 *方法描述：根据页面form传来的对象修改数据
	 *创建人：mail.zhangyong@gmail.com
	 *创建时间：2023-04-19 14:41:09
	 **/
	@PostMapping("/update")
	public ResultVo update(HttpServletResponse response,HttpServletRequest request) throws IOException{
		String categoryid = request.getParameter("categoryid");
		String processdefinitionid = request.getParameter("processdefinitionid");
		
		
		Category tab = this.categoryService.getById(categoryid);
		tab.setProcessdefinitionid(processdefinitionid);
		
		UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(request));
		tab.setUpdateuser(userToken.getLoginname());
		tab.setUpdatetime(System.currentTimeMillis());
		this.categoryService.updateById(tab);
		return ResultVoUtil.success();
	}

	/**
	*方法描述：通过主键获得对象信息
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2023-04-19 14:41:09
	**/
	@GetMapping("/getdata/{id}")
	@ResponseBody
	public ResultVo getdateCategory(HttpServletResponse response,HttpServletRequest request,@PathVariable("id") Long id) throws IOException{
		Category dto = this.categoryService.selectDtoById(id);
		if(null != dto){
			return ResultVoUtil.success(dto);
		}else{
			return ResultVoUtil.error();
		}
	}

	/**
	*方法描述：根据id字符串组删除对象方法，例如1,2,3
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2023-04-19 14:41:09
	**/
	@GetMapping("/delmore")
	@ResponseBody
	public ResultVo delMoreCategory(HttpServletResponse response,HttpServletRequest request) throws IOException{
		String ids = request.getParameter("id");
		this.categoryService.deleteByIdMore(ids);
		return ResultVoUtil.success();
	}

}
