package cn.jyjz.xiaoyao.oa.from.service.impl;

import cn.jyjz.xiaoyao.common.base.util.StringUtils;
import cn.jyjz.xiaoyao.common.redis.cache.ICacheManager;
import cn.jyjz.xiaoyao.oa.from.cache.CacheOaConstants;
import cn.jyjz.xiaoyao.oa.from.dataDao.CandidateMybatisDao;
import cn.jyjz.xiaoyao.oa.from.dataDao.CandidateuserMybatisDao;
import cn.jyjz.xiaoyao.oa.from.dataobject.Candidate;
import cn.jyjz.xiaoyao.oa.from.service.CandidateService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import cn.jyjz.xiaoyao.common.mybatisplus.base.BaseServiceImpl;
import lombok.extern.slf4j.Slf4j;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
* 业务层实现
*/
@Service
@Slf4j
public class CandidateServiceImpl extends BaseServiceImpl<CandidateMybatisDao, Candidate>  implements CandidateService {
	@Autowired
	private ICacheManager iCacheManager;
	@Autowired
	private CandidateMybatisDao candidateMybatisDao;
	/**
	 * 查询所有审批类型
	 * @return 返回list集合
	 */
	@Override
	public List<Candidate> listCandiate(){

		List<Candidate> list = iCacheManager.get(CacheOaConstants.OA_CANDIDATE_CACHE_KEY,CacheOaConstants.OA_CANDIDATE_CACHE_KEY);
		if(null == list || list.isEmpty()){
			synchronized (this){
				if(null == list || list.isEmpty()){
					list = this.list();
					iCacheManager.put(CacheOaConstants.OA_CANDIDATE_CACHE_KEY,CacheOaConstants.OA_CANDIDATE_CACHE_KEY,list);
				}
			}
		}
		return list;
	
	}
	
	/**
	 * 查询所有审批类型
	 * @return  返回map集合
	 */
	@Override
	public Map<Long,Candidate> mapCandiate(){
		List<Candidate> list = listCandiate();
		Map<Long,Candidate> map = list.stream().collect(Collectors.toMap(Candidate::getId,candidate -> candidate));
		
		return map;
	}
	
	/**
	 * 清理分类缓存
	 */
	@Override
	public void clearCache(){
		iCacheManager.remove(CacheOaConstants.OA_CANDIDATE_CACHE_KEY,CacheOaConstants.OA_CANDIDATE_CACHE_KEY);
	}
	
	/**
	 * 根据用户查询对应的审批角色
	 * @return 返回list集合
	 */
	public List<Candidate> listCandiateByUserid(Long userid){
		
		return this.candidateMybatisDao.selectListByUserid(userid);
	}
	
	/**
	 * 根据审批角色名称查询
	 * @return 返回list集合
	 */
	public List<Candidate> listCandiateByKey(String key){
		
		if(StringUtils.isNotBlank(key)){
			QueryWrapper queryWrapper = new QueryWrapper();
			queryWrapper.like("candidatename",key);
			return this.list(queryWrapper);
		}

		return listCandiate();
	}
}
