package cn.jyjz.xiaoyao.oa.from.service.impl;

import cn.jyjz.xiaoyao.admin.dataobject.Distionary;
import cn.jyjz.xiaoyao.admin.dataobject.User;
import cn.jyjz.xiaoyao.admin.service.DistionaryService;
import cn.jyjz.xiaoyao.admin.service.UserService;
import cn.jyjz.xiaoyao.common.base.util.StringUtils;
import cn.jyjz.xiaoyao.common.base.util.requestFormat.SearchQuery;
import cn.jyjz.xiaoyao.oa.from.dataDao.CandidateuserMybatisDao;
import cn.jyjz.xiaoyao.oa.from.dataobject.Candidateuser;
import cn.jyjz.xiaoyao.oa.from.dataobject.Userdistionary;
import cn.jyjz.xiaoyao.oa.from.service.CandidateuserService;
import cn.jyjz.xiaoyao.oa.from.service.UserdistionaryService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import cn.jyjz.xiaoyao.common.mybatisplus.base.BaseServiceImpl;
import lombok.extern.slf4j.Slf4j;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
* 业务层实现
*/
@Service
@Slf4j
public class CandidateuserServiceImpl extends BaseServiceImpl<CandidateuserMybatisDao, Candidateuser>  implements CandidateuserService {
	@Autowired
	private CandidateuserMybatisDao candidateusermybatisdao;
	
	@Autowired
	private UserdistionaryService userdistionaryService;
	
	@Autowired
	private DistionaryService distionaryService;
	@Autowired
	private UserService userService;
	/**
	 * 查询用户的关系对象
	 * @param candidateid 用户组主键
	 * @param userdisid 用户审批关系主键
	 * @return
	 */
	@Override
	public List<Candidateuser> selectList(String candidateid,Long userdisid ){
		
		SearchQuery searchQuery = new SearchQuery();
		if(StringUtils.isNotBlank(candidateid)){
			searchQuery.addEqual("candidateid",candidateid);
		}
		
		searchQuery.addEqual("userdisid",userdisid.toString());

		return candidateusermybatisdao.selectByCandidateuser(searchQuery.getQuery());
	}
	
	/**
	 * 查询用户的关系对象
	 * @param loginname 用户审批关系主键
	 * @return
	 */
	public List<Distionary> selectList(String loginname){
		//通过登录账号，查询所在审批关系插，查询oa_userdistionary_t表
		List<Userdistionary> list = userdistionaryService.selectListByLoginname(loginname);
		List<Distionary> distionaryList = new ArrayList<>();
		for (Userdistionary userdistionary:list){
			Distionary distionary = distionaryService.selectDistionaryById(userdistionary.getDeptidDis());
			distionaryList.add(distionary);
		}
		return distionaryList;
	}

	/**
	 * 根据用户关系，删除候选组关系
	 * @param userdisid 用户审批关系主键
	 * @return
	 */
	public boolean deleteByUserdisid(Long userdisid ){
		QueryWrapper queryWrapper = new QueryWrapper();
		queryWrapper.eq("USERDISID",userdisid);
		
		return (candidateusermybatisdao.delete(queryWrapper)>0)?true:false;
	}
	/**
	 * 返回可以流程的审批人员，返回登录账号
	 * @param deptid 审批部门主键
	 * @param candidateid 候选组主键
	 * @return 返回登录账号
	 */
	public String getClaimUserByDeptid(String deptid,String candidateid){
		List<Userdistionary> userdistionaryList = userdistionaryService.selectListByCandidateidAndDeptid(candidateid,deptid);
		//判断当前候选组是否存在审批人员
		if(null != userdistionaryList && !userdistionaryList.isEmpty()){
			//如果只有一个审批人直接返回
			if(userdistionaryList.size() == 1){
				User user = this.userService.selectDtoById(userdistionaryList.get(0).getUserid());
				if(null != user){
					return user.getLoginname();
				}
			}else{
				Random rand = new Random();
				Userdistionary userdistionary = userdistionaryList.get(rand.nextInt(userdistionaryList.size()));
				User user = this.userService.selectDtoById(userdistionary.getUserid());
				if(null != user){
					return user.getLoginname();
				}
			}
		}
		return null;
	}
	/**
	 * 返回可以流程的审批人员，返回登录账号
	 * @param loginname 流程发起人
	 * @param candidateid 候选组主键
	 * @return 返回登录账号
	 */
	public String getClaimUser(String loginname,String candidateid){
		
		//排除不在同一个店中的人员
		List<Userdistionary> userdistionaryList = userdistionaryService.selectListByCandidateid(candidateid,loginname);
		
		//判断当前候选组是否存在审批人员
		if(null != userdistionaryList && !userdistionaryList.isEmpty()){
		
			//如果只有一个审批人直接返回
			if(userdistionaryList.size() == 1){
				User user = this.userService.selectDtoById(userdistionaryList.get(0).getUserid());
				if(null != user){
					return user.getLoginname();
				}
			}else{
				
				//任务审批人员
				User user = null;
				
				//通过登录账号，查询所在审批关系插，查询oa_userdistionary_t表
				List<Userdistionary> list = userdistionaryService.selectListByLoginname(loginname);
				
				//以店面作为主键
				Map<Long,Userdistionary> userdistionaryMap = list.stream().collect(Collectors.toMap(Userdistionary::getDeptidDis, Function.identity(),(key1,key2) ->key2));
				//如果候选组有多个人，优先确定是否在同一个店面，
				for(Userdistionary userdistionary:userdistionaryList){
					if(null != userdistionaryMap.get(userdistionary.getDeptidDis())){
						//如果user不为空，说明有存在
						if(null != user){
							user = null;
							break;
						}
						user = this.userService.selectDtoById(userdistionary.getUserid());
					}
				}
				
				//如果user不为空，说明只有一个审批人员
				if(null != user){
					return user.getLoginname();
				}
				
				//以部门作为主键
				Map<Long,Userdistionary> userdistionaryDeptMap = list.stream().collect(Collectors.toMap(Userdistionary::getDeptid, Function.identity(),(key1,key2) ->key2));
				//如果在同一个店面，也是多个人优先选同部门
				for(Userdistionary userdistionary:userdistionaryList){
					if(null != userdistionaryDeptMap.get(userdistionary.getDeptid())){
						//如果user不为空，说明有存在
						if(null != user){
							user = null;
							break;
						}
						user = this.userService.selectDtoById(userdistionary.getUserid());
					}
				}
				//如果user不为空，说明只有一个审批人员
				if(null != user){
					return user.getLoginname();
				}
				//最后随机
				Random rand = new Random();
				Userdistionary userdistionary = userdistionaryList.get(rand.nextInt(userdistionaryList.size()));
				user = this.userService.selectDtoById(userdistionary.getUserid());
				//如果用户为空，再随机抽取一次
				if(null == user){
					user = this.userService.selectDtoById(userdistionary.getUserid());
				}
				//如果user不为空，说明只有一个审批人员
				if(null != user){
					return user.getLoginname();
				}
			}
		}
		
		return null;
	}
}
