package cn.jyjz.xiaoyao.oa.from.vo;


import cn.jyjz.xiaoyao.admin.dataobject.Department;
import cn.jyjz.xiaoyao.common.base.jsonDomain.DateSerializer;
import cn.jyjz.xiaoyao.common.base.util.requestFormat.SearchQuery;
import cn.jyjz.xiaoyao.common.mybatisplus.dto.BaseDto;
import cn.jyjz.xiaoyao.oa.from.dataobject.Candidate;
import cn.jyjz.xiaoyao.oa.from.dataobject.Userdistionary;
import com.baomidou.mybatisplus.annotation.TableField;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class UserDistionaryVo implements java.io.Serializable, BaseDto {
    
    private static final long serialVersionUID = 1523869402501829187L;
    @Schema(description = "用户流水号")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long id;
    
    @Schema(description = "登陆账号")
    private String loginname;
    
    @Schema(description = "用户名")
    private String username;
    
    @Schema(description = "该字段用于人员排序使用")
    private Integer sorting;
    
    @Schema(description = "是否被禁用，0禁用，1正常")
    private Integer enabled;
    
    @Schema(description = "性别，0：女，1：男")
    private String usex;
    
    @Schema(description = "描述")
    private String userdesc;
    
    @Schema(description = "手机号")
    private String mobile;
    
    @Schema(description = "电子邮箱")
    private String email;
    
    @Schema(description = "用户头像")
    private String usericon;
    
    @Schema(description = "用户来源，1：系统用户，2：油箱注册用户，3：第三方接入用户，其他待定")
    private Integer usertype;
    
    @Schema(description = "信息是否完善,默认为0:标示完善,1:待完善,2:信息有误(用户提供有误信息),3:用户身份证照片发送错误")
    private Integer perfect;
    
    @Schema(description = "注册来源,用于记录用户注册信息是从哪个站点来的")
    private Long fromsit;
    
    @Schema(description = "创建人，记录创建人的帐号")
    private String createuser;
    
    @Schema(description = "创建时间")
    @TableField(value="CREATEDATE")
    @JsonSerialize(using = DateSerializer.class)
    private Long createdate;
    
    @Schema(description = "修改人，记录创建人的帐号")
    private String updateuser;
    
    @Schema(description = "更新时间")
    @JsonSerialize(using = DateSerializer.class)
    private Long updatetime;
    
    @Schema(description = "密码最后更新时间")
    @JsonSerialize(using = DateSerializer.class)
    private Long psdupdatetime;
    
    //人员关系主键
    @JsonSerialize(using = ToStringSerializer.class)
    private long userdistionaryid;
    
    @Schema(description = "审批流节点是否跳过标志，默认为0，前端人员，1：后勤人员0：审批流所有节点，1：跳过店长审批")
    @TableField(value="USERTYPE")
    private Integer userOsudType;
    //审批组名称
    @TableField(exist = false)
    private String candidatename;
    
    //审批组主键
    @TableField(exist = false)
    private String candidateid;
    
    //保存当前登录用户的数据权限范围的搜索条件
    @TableField(exist = false)
    @JsonIgnore
    private SearchQuery searchQueryrolesShowleave;

    
    //用于记录用户对应的隶属部门对象,逻辑上做的一对一
    @JsonIgnore
    @TableField(exist = false)
    private Department department;
    
    //用户隶属部门主键，用户表中未做映射,逻辑上做的一对一
    @TableField(exist = false)
    @JsonSerialize(using = ToStringSerializer.class)
    private Long departmentId = 0L;
    
    //部门名称，列表使用
    @TableField(exist = false)
    private String departname = "系统";
    
    
    //用户对应的部门信息，映射文件中对应的字段信息
    @JsonIgnore
    @TableField(exist = false)
    private List<Department> deptlist = new ArrayList<Department>(0);
    
    @TableField(exist = false)
    private List<Candidate> candidateList = new ArrayList<Candidate>(0);
    
    public void setDeptlist(List<Department> deptlist) {
        this.formatDeptlist(deptlist);
    }
    
    public void formatDeptlist(List<Department> deptlist) {
        Iterator<Department> it = deptlist.iterator();
        StringBuffer sb = new StringBuffer();
        
        if (it.hasNext()) {
            Department dept = it.next();
            
            if (null != sb && sb.length() > 0) {
                sb.append("," + dept.getDepartname());
            }else{
                sb.append(dept.getDepartname());
            }
            
            if(null != dept){
                this.departmentId = dept.getId();
                department = dept;
            }
            
        }
        this.departname = sb.toString();
        
        this.deptlist = deptlist;
    }
    
    @TableField(exist = false)
    @JsonIgnore
    private final Map<String,String> query = Stream.of(
                    new AbstractMap.SimpleEntry<>("ID","id"),
                    new AbstractMap.SimpleEntry<>("LOGINNAME","loginname"),
                    new AbstractMap.SimpleEntry<>("PASSWORD","password"),
                    new AbstractMap.SimpleEntry<>("USERNAME","username"),
                    new AbstractMap.SimpleEntry<>("ISSYS","issys"),
                    new AbstractMap.SimpleEntry<>("SORTING","sorting"),
                    new AbstractMap.SimpleEntry<>("ENABLED","enabled"),
                    new AbstractMap.SimpleEntry<>("USEX","usex"),
                    new AbstractMap.SimpleEntry<>("USERDESC","userdesc"),
                    new AbstractMap.SimpleEntry<>("MOBILE","mobile"),
                    new AbstractMap.SimpleEntry<>("EMAIL","email"),
                    new AbstractMap.SimpleEntry<>("USERICON","usericon"),
                    new AbstractMap.SimpleEntry<>("USERTYPE","usertype"),
                    new AbstractMap.SimpleEntry<>("PERFECT","perfect"),
                    new AbstractMap.SimpleEntry<>("FROMSIT","fromsit"),
                    new AbstractMap.SimpleEntry<>("CREATEUSER","createuser"),
                    new AbstractMap.SimpleEntry<>("CREATEDATE","createdate"),
                    new AbstractMap.SimpleEntry<>("UPDATEUSER","updateuser"),
                    new AbstractMap.SimpleEntry<>("UPDATETIME","updatetime"),
                    new AbstractMap.SimpleEntry<>("listdept","uo.DEPARTID"),
                    new AbstractMap.SimpleEntry<>("PSDUPDATETIME","psdupdatetime")
            
            )
            .collect(Collectors.toMap(AbstractMap.SimpleEntry::getKey, AbstractMap.SimpleEntry::getValue));
    
    public String getQueryFiled(String filedname){
        String obj = null;
        if(null != query && query.size() > 0){
            obj = query.get(filedname);
        }
        return obj;
    }
    
}
