package cn.jyjz.xiaoyao.ocr.controller;

import java.io.IOException;
import java.util.Date;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.jyjz.xiaoyao.common.base.util.RequestBaseUtil;
import cn.jyjz.xiaoyao.common.base.vo.UserToken;
import cn.jyjz.xiaoyao.common.mybatisplus.util.SearchQueryFormat;
import cn.jyjz.xiaoyao.ocr.dataobject.OcrBooknote;
import cn.jyjz.xiaoyao.ocr.service.OcrBooknoteService;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;

import cn.jyjz.xiaoyao.admin.service.UserService;
import cn.jyjz.xiaoyao.common.base.service.UserTokenService;
import cn.jyjz.xiaoyao.common.base.controller.BaseController;
import cn.jyjz.xiaoyao.common.base.param.ParamterPage;
import cn.jyjz.xiaoyao.common.base.util.requestFormat.SearchQuery;
import cn.jyjz.xiaoyao.common.base.vo.ResultVo;
import cn.jyjz.xiaoyao.common.base.vo.ResultVoUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;

@Api(tags="ocr_个人笔记")
@RestController
@RequestMapping("/ocr/ocrBooknote")
public class OcrBooknoteController extends BaseController{

	private Logger log = LoggerFactory.getLogger(this.getClass());

	@Autowired
	private OcrBooknoteService ocrBooknoteService;

	@Autowired
	private UserTokenService userTokenService;
	@Autowired
	private UserService userService;
	
	/**
	 *   添加个人笔记
	 *
	 * @return
	 */
	@ApiOperation(value="添加个人笔记", notes="添加个人笔记")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "header", name = "X-Tenant-Id", value = "租户主键,用户登录时的信息“tenantList”中获得。", dataType = "String", required = true),
			@ApiImplicitParam(paramType = "path",name = "notecontent", value =  "笔记内容", required = true)
	})
	@PostMapping(value = "/add")
	public ResultVo<String> add(@RequestParam(name="notecontent",required=true) String notecontent,HttpServletRequest request) {
		
		//获取当前登录人
		UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(request));
		//租户主键，由前端页面传送
		String tenantId = request.getHeader("X-Tenant-Id");
		
		if(StringUtils.isBlank(tenantId)){
			return ResultVoUtil.error("租户主键不能为空");
		}
		
		QueryWrapper queryWrapper = new QueryWrapper();
		queryWrapper.eq("CREATEUSER",userToken.getUsername());
		queryWrapper.eq("TENANTID",tenantId);
		
		List<OcrBooknote> list = ocrBooknoteService.list(queryWrapper);
		
		if(null == list || list.isEmpty()){
			OcrBooknote ocrBooknote = new OcrBooknote();
			
			ocrBooknote.setCreateuser(userToken.getLoginname());
			ocrBooknote.setCreatedate(System.currentTimeMillis());
			ocrBooknote.setNotecontent(notecontent);
			ocrBooknote.setTenantid(Long.parseLong(tenantId));
			ocrBooknoteService.save(ocrBooknote);
		}else{
			OcrBooknote ocrBooknote = list.get(0);;
			ocrBooknote.setNotecontent(notecontent);
			ocrBooknoteService.updateById(ocrBooknote);
		}
		
		
		return ResultVoUtil.success("添加成功！");
	}
	
	/**
	 *   获取个人笔记
	 *
	 * @return
	 */
	@ApiOperation(value="获取个人笔记", notes="获取个人笔记")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "header", name = "X-Tenant-Id", value = "租户主键,用户登录时的信息“tenantList”中获得。", dataType = "String", required = true)
	})
	@GetMapping(value = "/query")
	public ResultVo<OcrBooknote> query(HttpServletRequest request) {
		//获取当前登录人
		UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(request));
		//租户主键，由前端页面传送
		String tenantId = request.getHeader("X-Tenant-Id");
		
		if(StringUtils.isBlank(tenantId)){
			return ResultVoUtil.error("租户主键不能为空");
		}
		
		QueryWrapper queryWrapper = new QueryWrapper();
		queryWrapper.eq("CREATEUSER",userToken.getLoginname());
		queryWrapper.eq("TENANTID",tenantId);
		
		List<OcrBooknote> list = ocrBooknoteService.list(queryWrapper);
		if(null == list || list.isEmpty()){
			return ResultVoUtil.success();
		}else{
			return ResultVoUtil.success(list.get(0));
		}
		
	}

}
