package cn.jyjz.xiaoyao.ocr.controller;

import java.io.IOException;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.jyjz.xiaoyao.common.base.util.RequestBaseUtil;
import cn.jyjz.xiaoyao.common.base.vo.UserToken;
import cn.jyjz.xiaoyao.common.mybatisplus.util.SearchQueryFormat;
import cn.jyjz.xiaoyao.monitor.service.server.Sys;
import cn.jyjz.xiaoyao.ocr.dataobject.OcrConfiguration;
import cn.jyjz.xiaoyao.ocr.service.OcrConfigurationService;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;

import cn.jyjz.xiaoyao.admin.service.UserService;
import cn.jyjz.xiaoyao.common.base.service.UserTokenService;
import cn.jyjz.xiaoyao.common.base.controller.BaseController;
import cn.jyjz.xiaoyao.common.base.param.ParamterPage;
import cn.jyjz.xiaoyao.common.base.util.requestFormat.SearchQuery;
import cn.jyjz.xiaoyao.common.base.vo.ResultVo;
import cn.jyjz.xiaoyao.common.base.vo.ResultVoUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;

@Api(tags="ocr-配置管理")
@RestController
@RequestMapping("/ocr/ocrConfiguration")
public class OcrConfigurationController extends BaseController{

	private Logger log = LoggerFactory.getLogger(this.getClass());

	@Autowired
	private OcrConfigurationService ocrConfigurationService;

	@Autowired
	private UserTokenService userTokenService;
	@Autowired
	private UserService userService;
	/**
	*方法描述：返回json字符串，接受参数，dto名称以及分页信息
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2024-01-14 10:27:01
	**/
	@GetMapping("/list")
	public ResultVo<IPage<OcrConfiguration>> listOcrConfiguration(ParamterPage paramterPage, HttpServletRequest request, HttpServletResponse response) throws IOException{
		SearchQuery searchQuery = this.getParametersStartingWithToSearchJson(request, "search_");

		OcrConfiguration ocrconfiguration = new OcrConfiguration();

		QueryWrapper<OcrConfiguration> queryWrapper = SearchQueryFormat.queryStringFormat(searchQuery,ocrconfiguration);
		IPage<OcrConfiguration> page = this.ocrConfigurationService.selectSearchListPage(paramterPage,queryWrapper);;
		return ResultVoUtil.success(page);
	}
	
	
	/**
	 * 分页列表查询
	 *
	 * @param request
	 * @return
	 */
	//@AutoLog(value = "ocr_configuration-分页列表查询")
	@ApiOperation(value="获取配置信息", notes="获取配置信息")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "header", name = "X-Tenant-Id", value = "租户主键,用户登录时的信息“tenantList”中获得。", dataType = "String", required = true)
	})
	@GetMapping(value = "/getConfiguration")
	public ResultVo<OcrConfiguration> getConfiguration(HttpServletRequest request) {
		
		//获取当前登录人
		UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(request));
		
		//租户主键，由前端页面传送
		String tenantId = request.getHeader("X-Tenant-Id");
		if(StringUtils.isBlank(tenantId)){
			return ResultVoUtil.error("租户主键不能为空");
		}
		OcrConfiguration ocrConfiguration = ocrConfigurationService.getConfiguration(userToken.getLoginname(),tenantId);
		if(null != ocrConfiguration){
			return ResultVoUtil.success(ocrConfiguration);
		}
		
		return ResultVoUtil.error("没有配置信息");
	}

	/**
	*方法描述：根据页面form传来的对象修改数据
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2024-01-14 10:27:01
	**/
	@PostMapping("/edit")
	public ResultVo editOcrConfiguration(HttpServletResponse response,HttpServletRequest request,OcrConfiguration tab) throws IOException{
		UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(request));
		tab.setUpdateby(userToken.getLoginname());
		tab.setUpdatetime(System.currentTimeMillis());
		this.ocrConfigurationService.updateById(tab);
		return ResultVoUtil.success();
	}

	/**
	*方法描述：通过主键获得对象信息
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2024-01-14 10:27:01
	**/
	@GetMapping("/getdata")
	@ResponseBody
	public ResultVo getdateOcrConfiguration(HttpServletResponse response,HttpServletRequest request) throws IOException{
		List<OcrConfiguration> list = this.ocrConfigurationService.list();

		if(null != list && !list.isEmpty()){
			return ResultVoUtil.success(list.get(0));
		}else{
			return ResultVoUtil.error();
		}
	}

}
