package cn.jyjz.xiaoyao.ocr.controller;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.jyjz.xiaoyao.admin.service.TenantService;
import cn.jyjz.xiaoyao.common.base.util.RequestBaseUtil;
import cn.jyjz.xiaoyao.common.base.vo.UserToken;
import cn.jyjz.xiaoyao.common.mybatisplus.util.SearchQueryFormat;
import cn.jyjz.xiaoyao.ocr.dataobject.OcrPicturetype;
import cn.jyjz.xiaoyao.ocr.dataobject.OcrPlan;
import cn.jyjz.xiaoyao.ocr.service.OcrPlanService;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;

import cn.jyjz.xiaoyao.admin.service.UserService;
import cn.jyjz.xiaoyao.common.base.service.UserTokenService;
import cn.jyjz.xiaoyao.common.base.controller.BaseController;
import cn.jyjz.xiaoyao.common.base.param.ParamterPage;
import cn.jyjz.xiaoyao.common.base.util.requestFormat.SearchQuery;
import cn.jyjz.xiaoyao.common.base.vo.ResultVo;
import cn.jyjz.xiaoyao.common.base.vo.ResultVoUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;

@RestController
@RequestMapping("/ocr/ocrPlan")
public class OcrPlanController extends BaseController{

	private Logger log = LoggerFactory.getLogger(this.getClass());

	@Resource
	private OcrPlanService ocrPlanService;

	@Resource
	private UserTokenService userTokenService;
	@Resource
	private UserService userService;
	@Resource
	private TenantService tenantService;
	/**
	 *方法描述：返回json字符串，接受参数，dto名称以及分页信息
	 *创建人：mail.zhangyong@gmail.com
	 *创建时间：2024-01-14 10:27:02
	 **/
	@ApiOperation(value="ocr_任务计划列表查询", notes="ocr_任务计划列表查询，无分页")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "header", name = "X-Tenant-Id", value = "租户主键,用户登录时的信息“tenantList”中获得。", dataType = "String", required = true)
	})
	@GetMapping("/listall")
	public ResultVo<List<OcrPlan>> listall(HttpServletRequest request, HttpServletResponse response) throws IOException{
		
		//租户主键，由前端页面传送
		String tenantId = request.getHeader("X-Tenant-Id");
		if(StringUtils.isBlank(tenantId)){
			return ResultVoUtil.error("租户主键不能为空");
		}
		
		SearchQuery searchQuery = this.getParametersStartingWithToSearchJson(request, "search_");
		OcrPlan ocrplan = new OcrPlan();
		QueryWrapper<OcrPlan> queryWrapper = SearchQueryFormat.queryStringFormat(searchQuery,ocrplan);
		queryWrapper.eq("TENANTID",tenantId);
		queryWrapper.orderByDesc("REORDER");
		List<OcrPlan> page = this.ocrPlanService.queryListNoPage(queryWrapper);;
		return ResultVoUtil.success(page);
	}
	
	/**
	*方法描述：返回json字符串，接受参数，dto名称以及分页信息
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2024-01-14 10:27:02
	**/
	@GetMapping("/list")
	public ResultVo<IPage<OcrPlan>> listOcrPlan(ParamterPage paramterPage, HttpServletRequest request, HttpServletResponse response) throws IOException{
		SearchQuery searchQuery = this.getParametersStartingWithToSearchJson(request, "search_");

		OcrPlan ocrplan = new OcrPlan();

		QueryWrapper<OcrPlan> queryWrapper = SearchQueryFormat.queryStringFormat(searchQuery,ocrplan);
		IPage<OcrPlan> page = this.ocrPlanService.selectSearchListPage(paramterPage,queryWrapper);
		for(OcrPlan ocrPlan:page.getRecords()){
			ocrPlan.setTenant(tenantService.selectDtoById(ocrPlan.getTenantid()));
		}
		return ResultVoUtil.success(page);
	}

	/**
	*方法描述：保存添加数据
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2024-01-14 10:27:02
	**/
	@PostMapping("/save")
	public ResultVo saveOcrPlan(HttpServletResponse response,HttpServletRequest request,OcrPlan tab) throws IOException{
		UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(request));
		tab.setCreateby(userToken.getLoginname());
		tab.setCreatetime(System.currentTimeMillis());
		this.ocrPlanService.save(tab);
		return ResultVoUtil.success();
	}

	/**
	*方法描述：根据页面form传来的对象修改数据
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2024-01-14 10:27:02
	**/
	@PostMapping("/edit")
	public ResultVo editOcrPlan(HttpServletResponse response,HttpServletRequest request,OcrPlan tab) throws IOException{
		UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(request));
		tab.setUpdateby(userToken.getLoginname());
		tab.setUpdatetime(System.currentTimeMillis());
		this.ocrPlanService.updateById(tab);
		return ResultVoUtil.success();
	}

	/**
	*方法描述：通过主键获得对象信息
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2024-01-14 10:27:02
	**/
	@GetMapping("/getdata/{id}")
	@ResponseBody
	public ResultVo getdateOcrPlan(HttpServletResponse response,HttpServletRequest request,@PathVariable("id") Long id) throws IOException{
		OcrPlan dto = this.ocrPlanService.selectDtoById(id);
		if(null != dto){
			return ResultVoUtil.success(dto);
		}else{
			return ResultVoUtil.error();
		}
	}

	/**
	*方法描述：根据id字符串组删除对象方法，例如1,2,3
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2024-01-14 10:27:02
	**/
	@GetMapping("/delmore")
	@ResponseBody
	public ResultVo delMoreOcrPlan(HttpServletResponse response,HttpServletRequest request) throws IOException{
		String ids = request.getParameter("id");
		this.ocrPlanService.deleteByIdMore(ids);
		return ResultVoUtil.success();
	}

}
