package cn.jyjz.xiaoyao.ocr.controller;

import java.io.IOException;
import java.util.List;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.jyjz.xiaoyao.common.base.util.RequestBaseUtil;
import cn.jyjz.xiaoyao.common.base.vo.UserToken;
import cn.jyjz.xiaoyao.common.mybatisplus.util.SearchQueryFormat;
import cn.jyjz.xiaoyao.ocr.dataobject.OcrSearchmanager;
import cn.jyjz.xiaoyao.ocr.service.OcrSearchmanagerService;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;

import cn.jyjz.xiaoyao.admin.service.UserService;
import cn.jyjz.xiaoyao.common.base.service.UserTokenService;
import cn.jyjz.xiaoyao.common.base.controller.BaseController;
import cn.jyjz.xiaoyao.common.base.param.ParamterPage;
import cn.jyjz.xiaoyao.common.base.util.requestFormat.SearchQuery;
import cn.jyjz.xiaoyao.common.base.vo.ResultVo;
import cn.jyjz.xiaoyao.common.base.vo.ResultVoUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;

@Api(tags="ocr-自定义筛选")
@RestController
@RequestMapping("/ocr/ocrSearchmanager")
public class OcrSearchmanagerController extends BaseController{

	private Logger log = LoggerFactory.getLogger(this.getClass());

	@Autowired
	private OcrSearchmanagerService ocrSearchmanagerService;

	@Autowired
	private UserTokenService userTokenService;
	@Autowired
	private UserService userService;
	
	/**
	 *   添加
	 * @param ocrSearchmanager
	 * @return
	 */
	@ApiOperation(value="筛选设置", notes="筛选设置")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "header", name = "X-Tenant-Id", value = "租户主键,用户登录时的信息“tenantList”中获得。", dataType = "String", required = true)
	})
	@PostMapping(value = "/add")
	public ResultVo<String> add(@RequestBody OcrSearchmanager ocrSearchmanager,HttpServletRequest req) {
		
		//获取当前登录人
		UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(req));
		
		//租户主键，由前端页面传送
		String tenantId = req.getHeader("X-Tenant-Id");
		if(StringUtils.isBlank(tenantId)){
			return ResultVoUtil.error("租户主键不能为空");
		}
		
		QueryWrapper queryWrapper = new QueryWrapper();
		queryWrapper.eq("create_by",userToken.getLoginname());
		queryWrapper.eq("tenant_id",tenantId);
		
		List<OcrSearchmanager> list =  ocrSearchmanagerService.list(queryWrapper);
		
		if(null != list && !list.isEmpty()) {
			List<Long> listids = list.stream().map(OcrSearchmanager::getId).collect(Collectors.toList());
			ocrSearchmanagerService.removeByIds(listids);
		}
		ocrSearchmanager.setCreateBy(userToken.getLoginname());
		ocrSearchmanager.setCreateTime(System.currentTimeMillis());
		ocrSearchmanager.setTenantId(Long.parseLong(tenantId));
		
		return ocrSearchmanagerService.save(ocrSearchmanager)?ResultVoUtil.success("添加成功！"):ResultVoUtil.error();
	}
	
	
	/**
	 * 通过当前用户查询
	 *
	 * @return
	 */
	@ApiOperation(value="自定义筛选设置查询", notes="自定义筛选设置查询")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "header", name = "X-Tenant-Id", value = "租户主键,用户登录时的信息“tenantList”中获得。", dataType = "String", required = true)
	})
	@GetMapping(value = "/query")
	public ResultVo<OcrSearchmanager> queryById(HttpServletRequest req) {
		
		//获取当前登录人
		UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(req));
		
		//租户主键，由前端页面传送
		String tenantId = req.getHeader("X-Tenant-Id");
		if(StringUtils.isBlank(tenantId)){
			return ResultVoUtil.error("租户主键不能为空");
		}
		
		QueryWrapper queryWrapper = new QueryWrapper();
		queryWrapper.eq("create_by",userToken.getLoginname());
		queryWrapper.eq("tenant_id",tenantId);
		
		OcrSearchmanager ocrSearchmanager = null;
		List<OcrSearchmanager> list =  ocrSearchmanagerService.list(queryWrapper);
		
		if(null != list && !list.isEmpty()) {
			ocrSearchmanager = list.get(0);
		}
		return ResultVoUtil.success(ocrSearchmanager);
	}
}
