package cn.jyjz.xiaoyao.ocr.controller;

import java.io.IOException;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import cn.jyjz.xiaoyao.common.base.util.RequestBaseUtil;
import cn.jyjz.xiaoyao.common.base.vo.UserToken;
import cn.jyjz.xiaoyao.common.mybatisplus.util.SearchQueryFormat;
import cn.jyjz.xiaoyao.ocr.dataobject.OcrPicturetype;
import cn.jyjz.xiaoyao.ocr.dataobject.OcrUpuser;
import cn.jyjz.xiaoyao.ocr.service.OcrUpuserService;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;

import cn.jyjz.xiaoyao.admin.service.UserService;
import cn.jyjz.xiaoyao.common.base.service.UserTokenService;
import cn.jyjz.xiaoyao.common.base.controller.BaseController;
import cn.jyjz.xiaoyao.common.base.param.ParamterPage;
import cn.jyjz.xiaoyao.common.base.util.requestFormat.SearchQuery;
import cn.jyjz.xiaoyao.common.base.vo.ResultVo;
import cn.jyjz.xiaoyao.common.base.vo.ResultVoUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;

@Api(tags="ocr_提报人管理")
@RestController
@RequestMapping("/ocr/ocrUpuser")
public class OcrUpuserController extends BaseController{

	private Logger log = LoggerFactory.getLogger(this.getClass());

	@Autowired
	private OcrUpuserService ocrUpuserService;

	@Autowired
	private UserTokenService userTokenService;
	@Autowired
	private UserService userService;
	
	
	/**
	 * 提报人列表查询
	 *
	 * @return
	 */
	//@AutoLog(value = "ocr_upuser-分页列表查询")
	@ApiOperation(value="ocr_提报人查询", notes="ocr_提报人查询，无分页")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "header", name = "X-Tenant-Id", value = "租户主键,用户登录时的信息“tenantList”中获得。", dataType = "String", required = true)
	})
	@GetMapping(value = "/listall")
	public ResultVo<IPage<OcrUpuser>> queryPageList(
												  HttpServletRequest req) {
		//租户主键，由前端页面传送
		String tenantId = req.getHeader("X-Tenant-Id");
		if(StringUtils.isBlank(tenantId)){
			return ResultVoUtil.error("租户主键不能为空");
		}
		
		SearchQuery searchQuery = this.getParametersStartingWithToSearchJson(req, "search_");
		OcrUpuser ocrUpuser = new OcrUpuser();
		QueryWrapper<OcrUpuser> queryWrapper = SearchQueryFormat.queryStringFormat(searchQuery,ocrUpuser);
		queryWrapper.eq("tenant_id",tenantId);
		queryWrapper.orderByAsc("convert(upname using gbk)");
		
		List<OcrUpuser> pageList = ocrUpuserService.queryListNoPage(queryWrapper);
		return ResultVoUtil.success(pageList);
	}
	
	/**
	*方法描述：返回json字符串，接受参数，dto名称以及分页信息
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2024-01-14 10:27:02
	**/
	@GetMapping("/list")
	public ResultVo<IPage<OcrUpuser>> listOcrUpuser(ParamterPage paramterPage, HttpServletRequest request, HttpServletResponse response) throws IOException{
		SearchQuery searchQuery = this.getParametersStartingWithToSearchJson(request, "search_");

		OcrUpuser ocrupuser = new OcrUpuser();

		QueryWrapper<OcrUpuser> queryWrapper = SearchQueryFormat.queryStringFormat(searchQuery,ocrupuser);
		IPage<OcrUpuser> page = this.ocrUpuserService.selectSearchListPage(paramterPage,queryWrapper);;
		return ResultVoUtil.success(page);
	}

	/**
	*方法描述：保存添加数据
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2024-01-14 10:27:02
	**/
	@PostMapping("/save")
	public ResultVo saveOcrUpuser(HttpServletResponse response,HttpServletRequest request,OcrUpuser tab) throws IOException{
		UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(request));
		tab.setCreateBy(userToken.getLoginname());
		tab.setCreateTime(System.currentTimeMillis());
		this.ocrUpuserService.save(tab);
		return ResultVoUtil.success();
	}

	/**
	*方法描述：根据页面form传来的对象修改数据
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2024-01-14 10:27:02
	**/
	@PostMapping("/edit")
	public ResultVo editOcrUpuser(HttpServletResponse response,HttpServletRequest request,OcrUpuser tab) throws IOException{
		UserToken userToken = this.userTokenService.getUserToken(RequestBaseUtil.getToken(request));
		tab.setUpdateBy(userToken.getLoginname());
		tab.setUpdateTime(System.currentTimeMillis());
		this.ocrUpuserService.updateById(tab);
		return ResultVoUtil.success();
	}

	/**
	*方法描述：通过主键获得对象信息
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2024-01-14 10:27:02
	**/
	@GetMapping("/getdata/{id}")
	@ResponseBody
	public ResultVo getdateOcrUpuser(HttpServletResponse response,HttpServletRequest request,@PathVariable("id") Long id) throws IOException{
		OcrUpuser dto = this.ocrUpuserService.selectDtoById(id);
		if(null != dto){
			return ResultVoUtil.success(dto);
		}else{
			return ResultVoUtil.error();
		}
	}

	/**
	*方法描述：根据id字符串组删除对象方法，例如1,2,3
	*创建人：mail.zhangyong@gmail.com
	*创建时间：2024-01-14 10:27:02
	**/
	@GetMapping("/delmore")
	@ResponseBody
	public ResultVo delMoreOcrUpuser(HttpServletResponse response,HttpServletRequest request) throws IOException{
		String ids = request.getParameter("id");
		this.ocrUpuserService.deleteByIdMore(ids);
		return ResultVoUtil.success();
	}

}
