package cn.jyjz.xiaoyao.ocr.util.httputil;


import cn.jyjz.xiaoyao.ocr.util.ImageClassUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import okhttp3.*;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;

import java.io.IOException;
import java.util.Map;
import java.util.Set;

/**
 * @author zhangyong
 * @title: HttpClient
 * @projectName jyjz-parent
 * @description: TODO
 * @date 2022/3/18 3:49 PM
 */
public class HttpClient {

    private static Logger log = LoggerFactory.getLogger(HttpClient.class);

    public static final String DEFAULT_CHARSET = "UTF-8";
    public static final String JSON_CONTENT_FORM = "application/json;charset=UTF-8";
    public static final String CONTENT_FORM = "application/x-www-form-urlencoded;charset=UTF-8";

    public static final Integer HTTP_SUCCESS = 200;

    public static String doService(String url, HttpParamers paramers, HttpHeader header, int connectTimeout, int readTimeout) throws Exception {
        HttpMethod httpMethod = paramers.getHttpMethod();
        switch (httpMethod) {
            case GET:
                return doGet(url, paramers, header, connectTimeout, readTimeout);
            case POST:
                return doPost(url, paramers, header, connectTimeout, readTimeout);
        }
        return null;
    }

    /**
     * post方法
     * @param url
     * @param paramers
     * @param header
     * @param connectTimeout
     * @param readTimeout
     * @return
     * @throws IOException
     */
    public static String doPost(String url, HttpParamers paramers, HttpHeader header, int connectTimeout, int readTimeout) throws IOException  {
        String responseData = "";
        CloseableHttpClient httpClient = null;
        CloseableHttpResponse httpResponse = null;
        RequestConfig requestConfig = RequestConfig.custom()
                .setConnectTimeout(connectTimeout)
                .setConnectionRequestTimeout(connectTimeout)
                .setSocketTimeout(readTimeout).build();
        try{
            String query = null;
            HttpPost httpPost = new HttpPost(url);

            httpPost.setConfig(requestConfig);
            
            setHeader(httpPost, header);
            if (paramers.isJson()) {
                //json数据
                httpPost.setHeader(HTTP.CONTENT_TYPE, JSON_CONTENT_FORM);
                query = paramers.getJsonParamer();
            } else {
                //表单数据
                httpPost.setHeader(HTTP.CONTENT_TYPE, CONTENT_FORM);
                if(null != paramers){
                    query = paramers.getQueryString(DEFAULT_CHARSET);
                }
            }
            if(StringUtils.isNotBlank(query)){
                HttpEntity reqEntity = new StringEntity(query);
                httpPost.setEntity(reqEntity);
            }
            httpClient = HttpClients.createDefault();
            httpResponse = httpClient.execute(httpPost);
            HttpEntity resEntity = httpResponse.getEntity();
            responseData = EntityUtils.toString(resEntity);
        } finally{
            if(null != httpResponse){
                httpResponse.close();
            }
            if(null != httpClient){
                httpClient.close();
            }
        }
        return responseData;
    }


    /**
     * get方法
     * @param url
     * @param params
     * @param header
     * @param connectTimeout
     * @param readTimeout
     * @return
     * @throws IOException
     */
    public static String doGet(String url, HttpParamers params, HttpHeader header, int connectTimeout, int readTimeout) throws IOException {
        String responseData = "";
        CloseableHttpClient httpClient = null;
        CloseableHttpResponse httpResponse = null;
        RequestConfig requestConfig = RequestConfig.custom()
                .setConnectTimeout(connectTimeout)
                .setConnectionRequestTimeout(connectTimeout)
                .setSocketTimeout(readTimeout).build();
        try{
            String query = params.getQueryString(DEFAULT_CHARSET);
            url = buildGetUrl(url, query);
            HttpGet httpGet = new HttpGet(url);
    

            httpGet.setConfig(requestConfig);
            
            setHeader(httpGet, header);
            httpClient = HttpClients.createDefault();
            httpResponse = httpClient.execute(httpGet);
            HttpEntity resEntity = httpResponse.getEntity();
            responseData = EntityUtils.toString(resEntity);
        } finally{
            if(null != httpResponse){
                httpResponse.close();
            }

            httpClient.close();
        }
        return responseData;
    }

    private static void setHeader(HttpRequestBase httpRequestBase, HttpHeader header){
        if(header != null){
            Map<String,String> headerMap = header.getParams();
            if (headerMap != null && !headerMap.isEmpty()) {
                Set<Map.Entry<String, String>> entries = headerMap.entrySet();
                for (Map.Entry<String, String> entry : entries) {
                    String name = entry.getKey();
                    String value = entry.getValue();
                    httpRequestBase.setHeader(name, value);
                }
            }
        }
    }

    private static String buildGetUrl(String url, String query) throws IOException {
        if (query == null || query.equals("")) {
            return url;
        }
        StringBuilder newUrl = new StringBuilder(url);
        boolean hasQuery = url.contains("?");
        boolean hasPrepend = (url.endsWith("?")) || (url.endsWith("&"));
        if (!hasPrepend) {
            if (hasQuery) {
                newUrl.append("&");
            } else {
                newUrl.append("?");
                hasQuery = true;
            }
        }
        newUrl.append(query);
        hasPrepend = false;
        return newUrl.toString();
    }

    public static void main(String[] args){
    
//        JSONObject jsonObject = new JSONObject();
//        jsonObject.put("taskId","250872482880291103");
//        JSONArray jsonArray = new JSONArray();
//        jsonArray.add("https://copyright.bdstatic.com/vcg/creative/cc9c744cf9f7c864889c563cbdeddce6.jpg");
//        jsonArray.add("https://pic.rmb.bdstatic.com/bjh/914b8c0f9814b14c5fedeec7ec6615df5813.jpeg");
//        jsonObject.put("imgUrls",jsonArray);
//
//        log.info("jsonObject.toJSONString()={}",jsonObject.toJSONString());
//        RequestBody body = RequestBody.create(MediaType.parse("application/json"),jsonObject.toJSONString());
//
//        OkHttpClient client = new OkHttpClient().newBuilder()
//                .build();
//
//        Request request = new Request.Builder()
//                .url("http://47.93.3.37/api/classify/")
//                .method("POST", body)
//                .build();
//
//        Response response = null;
//        try {
//            response = client.newCall(request).execute();
//
//            log.info("token={}",response);
//            String string = response.body().string();
//            log.info("string={}",string);
//
//        } catch (IOException e) {
//            throw new RuntimeException(e);
//        }
//
    

//        //相似度检查
//        //https://console-docs.apipost.cn/preview/b85e3fe97510f177/96421d73df6c7f7d
//        JSONObject jsonObjectXs = new JSONObject();
//        jsonObjectXs.put("imgUrl","https://copyright.bdstatic.com/vcg/creative/cc9c744cf9f7c864889c563cbdeddce6.jpg");
//        jsonObjectXs.put("baseUrl","https://pic.rmb.bdstatic.com/bjh/914b8c0f9814b14c5fedeec7ec6615df5813.jpeg");
//
//        //接口文档
//        //https://console-docs.apipost.cn/preview/7add2088a9a191d5/874020cbfba4287b
//        String urlXs = "http://47.93.3.37/api/similarity/";
//        HttpParamers paramersXs = HttpParamers.httpPostParamers();
//        paramersXs.setJsonParamer(jsonObjectXs.toJSONString());
//        HttpHeader headerXs = null;
//        String responseDataXs = "";
//        try {
//            responseDataXs = HttpClient.doService(urlXs, paramersXs, headerXs, 15000, 30000);
//        } catch (Exception e) {
//
//        }{"imgUrl":"http://pic1.win4000.com/wallpaper/3/57c4f76f7b166.jpg","baseUrl":"http://pic1.win4000.com/wallpaper/3/57c4f76f7b166.jpg"}"
        JSONObject jsonObject = ImageClassUtil.getSimilarity("http://lmg.jj20.com/up/allimg/4k/s/02/210924231A341I-0-lp.jpg",
                "http://pic1.win4000.com/wallpaper/2/5461b9a4d1e09.jpg",
                null);
//        JSONObject jsonObject = ImageClassUtil.getSimilarity("https://pic.rmb.bdstatic.com/bjh/914b8c0f9814b14c5fedeec7ec6615df5813.jpeg",
//                "https://img2.baidu.com/it/u=3895999535,621145085&fm=253&fmt=auto&app=138&f=JPEG",
//                null);
        System.out.println(jsonObject.toJSONString());
        System.out.println("responseDataXs--code = [" + jsonObject.get("code") + "]");
        System.out.println("responseDataXs--similarity = [" + jsonObject.get("similarity") + "]");

        //类型检查
//        JSONObject jsonObjectVi = new JSONObject();
//        jsonObjectVi.put("taskId","250872482880291103");
//        JSONArray jsonArrayVi = new JSONArray();
//        jsonArrayVi.add("https://copyright.bdstatic.com/vcg/creative/cc9c744cf9f7c864889c563cbdeddce6.jpg");
//        jsonArrayVi.add("https://pic.rmb.bdstatic.com/bjh/914b8c0f9814b14c5fedeec7ec6615df5813.jpeg");
//        jsonObjectVi.put("imgUrls",jsonArrayVi);
//        //接口文档
//        //https://console-docs.apipost.cn/preview/7add2088a9a191d5/874020cbfba4287b
//        String urlVi = "http://47.93.3.37/api/classify/";
//        HttpParamers paramersVi = HttpParamers.httpPostParamers();
//        paramersVi.setJsonParamer(jsonObjectVi.toJSONString());
//        HttpHeader headerVi = null;
//        String responseDataVi = "";
//        try {
//            responseDataVi = HttpClient.doService(urlVi, paramersVi, headerVi, 15000, 30000);
//        } catch (Exception e) {
//
//        }
//        JSONObject jsonObjectd = ImageClassUtil.getClassify("https://copyright.bdstatic.com/vcg/creative/cc9c744cf9f7c864889c563cbdeddce6.jpg",
//                "250872482880291103",
//                null);
//
//        System.out.println("responseDatavi--code = [" + jsonObjectd.get("code") + "]");
//
//        JSONArray jsonArray = jsonObjectd.getJSONArray("data");
//
//        System.out.println("responseDatavi--classId = [" + jsonArray.getJSONObject(0).get("classId") + "]");
//        System.out.println("responseDatavi--scores = [" + jsonArray.getJSONObject(0).get("scores") + "]");
//        System.out.println(jsonObjectd.toJSONString());
//
//        Calendar rightNow = Calendar.getInstance();
//
//        rightNow.add(Calendar.MONTH,-3);//日期加3个月
//
//
//        System.out.println("startTime = [" + rightNow.getTime().getTime() + "]");

        //进行加密
//        BCryptPasswordEncoder encoder =new BCryptPasswordEncoder();
//        String temp = encoder.encode("18618432839");
//        System.out.println("18618432839 = [" + temp +"]");
//        System.out.println("bd-xxl = [" + encoder.encode("bd-xxl")  +"]");
//        System.out.println("ww01 = [" + encoder.encode("ww01")  +"]");
//        System.out.println("ds02-23 = [" + encoder.encode("ds02-23")  +"]");
//        System.out.println("百度原生-4 = [" + encoder.encode("百度原生-4")  +"]");
//
//        System.out.println("bdss-01 = [" + encoder.encode("bdss-01")  +"]");
//
//        System.out.println("bdxxl-01 = [" + encoder.encode("bdxxl-01")  +"]");
//
//        System.out.println("bdxxl-02 = [" + encoder.encode("bdxxl-02")  +"]");
//
//        boolean ispass = encoder.matches("ds01",temp);
//
//        System.out.println("ds01 --vaildate = [" + ispass);
//
//
//        String sss = "qingjiashenqingkey:203371905252589861:leaveService";
//
//        String[] s = sss.split(":");
//        System.out.println("s = " + Arrays.toString(s));
    }
}