package org.flowable.ui.common.security;


import java.util.Iterator;

import cn.jyjz.xiaoyao.admin.service.UserService;
import cn.jyjz.xiaoyao.common.base.service.UserTokenService;
import cn.jyjz.xiaoyao.common.base.util.RequestBaseUtil;
import cn.jyjz.xiaoyao.common.base.util.SpringUtils;
import cn.jyjz.xiaoyao.framework.security.model.AdminUserModel;
import org.apache.commons.lang3.StringUtils;
import org.flowable.idm.api.User;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

public class SecurityUtils {
    private static User assumeUser;

    private SecurityUtils() {
    }
    public static String getCurrentUserId() {
        UserTokenService userTokenService = SpringUtils.getBean(UserTokenService.class);
        HttpServletRequest request = ((ServletRequestAttributes) (RequestContextHolder.currentRequestAttributes())).getRequest();
        String token = RequestBaseUtil.getToken(request);
        if(StringUtils.isBlank(token)){
            return null;
        }
        Object object = userTokenService.getSessionAdminUserModelFromCache(request);
        if(null == object){
            object = SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        }
        if(null != object && object instanceof AdminUserModel){
            AdminUserModel adminUserModel = (AdminUserModel) object;
            FlowableAppUser appUser = (FlowableAppUser) adminUserModel.getFlowableUser();
    
            //User user = appUser.getUserObject();
            return appUser != null ? appUser.getUsername() : null;
        }
        return null;
    }
    
    public static User getCurrentUserObject() {
        if (assumeUser != null) {
            return assumeUser;
        } else {
            User user = null;
            UserTokenService userTokenService = SpringUtils.getBean(UserTokenService.class);
            HttpServletRequest request = ((ServletRequestAttributes) (RequestContextHolder.currentRequestAttributes())).getRequest();
            String token = RequestBaseUtil.getToken(request);
    
            Object object = null;
            if(StringUtils.isNotBlank(token)){
                object = userTokenService.getSessionAdminUserModelFromCache(request);
            }
            if(null == object){
                object = SecurityContextHolder.getContext().getAuthentication().getPrincipal();
            }
            if(null != object && object instanceof AdminUserModel){
                AdminUserModel adminUserModel = (AdminUserModel) object;
                FlowableAppUser appUser = (FlowableAppUser) adminUserModel.getFlowableUser();
                if (appUser != null) {
                    user = appUser.getUserObject();
                }
            }

            
            return user;
        }
    }
    
    public static FlowableAppUser getCurrentFlowableAppUser() {
        FlowableAppUser user = null;
        UserTokenService userTokenService = SpringUtils.getBean(UserTokenService.class);
        HttpServletRequest request = ((ServletRequestAttributes) (RequestContextHolder.currentRequestAttributes())).getRequest();
        String token = RequestBaseUtil.getToken(request);
        if(StringUtils.isBlank(token)){
            return null;
        }
        Object object = userTokenService.getSessionAdminUserModelFromCache(request);
        if(null == object){
            object = SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        }
        if(null != object && object instanceof AdminUserModel){
            AdminUserModel adminUserModel = (AdminUserModel) object;
            user = (FlowableAppUser) adminUserModel.getFlowableUser();
        }
        
        return user;
    }
    
    public static boolean currentUserHasCapability(String capability) {
        UserTokenService userTokenService = SpringUtils.getBean(UserTokenService.class);
        HttpServletRequest request = ((ServletRequestAttributes) (RequestContextHolder.currentRequestAttributes())).getRequest();
        String token = RequestBaseUtil.getToken(request);
        if(StringUtils.isBlank(token)){
            return false;
        }
        Object object = userTokenService.getSessionAdminUserModelFromCache(request);
        if(null == object){
            object = SecurityContextHolder.getContext().getAuthentication().getPrincipal();
        }
        
        if(null != object && object instanceof AdminUserModel){
            AdminUserModel adminUserModel = (AdminUserModel) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
            FlowableAppUser user = (FlowableAppUser) adminUserModel.getFlowableUser();
            Iterator var2 = user.getAuthorities().iterator();
    
            GrantedAuthority grantedAuthority;
            do {
                if (!var2.hasNext()) {
                    return false;
                }
        
                grantedAuthority = (GrantedAuthority)var2.next();
            } while(!capability.equals(grantedAuthority.getAuthority()));
        }
        
        return true;
    }
    
    public static void assumeUser(User user) {
        assumeUser = user;
    }
    
    public static void clearAssumeUser() {
        assumeUser = null;
    }
}